@@ -1,13 +1,13 @@
 # flake8: noqa
 import gretel_client._hybrid.aws as aws_hybrid
-import gretel_client._hybrid.azure as azure_hybrid
 import gretel_client._hybrid.gcp as gcp_hybrid
+import gretel_client._hybrid.azure as azure_hybrid
 
 from gretel_client._hybrid.config import configure_hybrid_session
 from gretel_client.config import ClientConfig, configure_session
 from gretel_client.evaluation.quality_report import QualityReport
 from gretel_client.gretel.interface import Gretel
-from gretel_client.helpers import poll, submit_docker_local
+from gretel_client.helpers import poll
 from gretel_client.projects.projects import (
     create_or_get_unique_project,
     create_project,
