@@ -14,21 +14,21 @@ import click
 
 from gretel_client.config import (
     ClientConfig,
-    configure_custom_logger,
-    get_session_config,
     RunnerMode,
+    add_session_context,
+    configure_custom_logger,
 )
-from gretel_client.models.config import get_status_description, StatusDescriptions
+from gretel_client.models.config import StatusDescriptions, get_status_description
 from gretel_client.projects.common import WAIT_UNTIL_DONE
-from gretel_client.projects.docker import check_docker_env
-from gretel_client.projects.exceptions import DockerEnvironmentError
 from gretel_client.projects.jobs import Job, WaitTimeExceeded
 from gretel_client.projects.models import Model
-from gretel_client.projects.projects import get_project, Project
+from gretel_client.projects.projects import Project, get_project
 from gretel_client.rest.exceptions import ApiException
 
 ExT = Union[str, Exception]
 
+CLI_SESSION_METADATA = {"cli": "1"}
+
 
 _copyright_data = """
 The Gretel CLI and Python SDK, installed through the "gretel-client"
@@ -318,7 +318,6 @@ def _determine_runner_mode(
 
 
 class SessionContext(object):
-
     _project: Optional[Project] = None
     """The project to use for this command (explicitly specified)."""
 
@@ -341,7 +340,9 @@ class SessionContext(object):
         self.debug = debug
         self.verbosity = 0
         self.output_fmt = output_fmt
-        self.config = session or get_session_config()
+        self.config = add_session_context(
+            session=session, client_metrics=CLI_SESSION_METADATA
+        )
         self.log = Logger(self.debug)
         configure_custom_logger(self.log)
         self.ctx = ctx
@@ -378,12 +379,7 @@ class SessionContext(object):
             )
 
             if self._runner == RunnerMode.LOCAL:
-                try:
-                    check_docker_env()
-                except DockerEnvironmentError as ex:
-                    raise DockerEnvironmentError(
-                        "Runner is local, but docker is not running. Please check that docker is installed and running."
-                    ) from ex
+                raise Exception("RunnerMode 'local' no longer supported")
 
     @property
     def session(self) -> ClientConfig:
@@ -657,7 +653,6 @@ def poll_and_print(
     callback: Callable = None,
 ):
     try:
-
         for update in job.poll_logs_status(wait=wait, callback=callback):
             if update.transitioned:
                 sc.log.info(
