@@ -1,30 +1,28 @@
 import json
 
 from pathlib import Path
-from typing import Tuple, Union
+from typing import Optional
 
 import click
 
 from gretel_client.cli.common import (
+    SessionContext,
     model_option,
     pass_session,
     poll_and_print,
     project_option,
     ref_data_option,
     runner_option,
-    SessionContext,
 )
-from gretel_client.cli.utils.parser_utils import ref_data_factory, RefData
-from gretel_client.models.config import get_model_type_config, GPU
-from gretel_client.projects.common import ModelArtifact, WAIT_UNTIL_DONE
-from gretel_client.projects.docker import ContainerRun, ContainerRunError
+from gretel_client.cli.utils.parser_utils import ref_data_factory
+from gretel_client.models.config import GPU, get_model_type_config
+from gretel_client.projects.common import WAIT_UNTIL_DONE, ModelArtifact
 from gretel_client.projects.jobs import Status
 from gretel_client.projects.models import Model, RunnerMode
 
 
 @click.group(help="Commands for training and working with models.")
-def models():
-    ...
+def models(): ...
 
 
 @models.command(help="Create a new model.")
@@ -169,46 +167,12 @@ def create(
     if dry_run:
         sc.exit(0)
 
-    # Start a local container when --runner is LOCAL
-    #
-    # The submit call above will have triggered the model to have been created in manual mode
-    # so at this point the model instance is hydrated with the data from the Cloud API
-    run = None
-    if runner == RunnerMode.LOCAL.value:
-        run = ContainerRun.from_job(model)
-        if sc.debug:
-            sc.log.debug("Enabling debug logs for the local container.")
-            run.enable_debug()
-        if output:
-            run.configure_output_dir(output)
-        if model.external_data_source:
-            run.configure_input_data(model.data_source)
-        if model.external_ref_data:
-            run.configure_ref_data(model.ref_data)
-        if upload_model:
-            sc.log.info("Uploads to Gretel Cloud are enabled.")
-            run.enable_cloud_uploads()
-        if model.instance_type == GPU:
-            sc.log.info("Configuring GPU for model training.")
-            try:
-                run.configure_gpu()
-                sc.log.info("GPU device found!")
-            except ContainerRunError:
-                sc.log.warn("Could not configure GPU. Continuing with CPU.")
-        sc.log.info(
-            f"This model is configured to run locally using the container {run.image}."
-        )
-        sc.log.info("Pulling the container and starting local model training.")
-        run.start()
-        sc.register_cleanup(lambda: run.graceful_shutdown())  # type:ignore
-
     # Poll for the latest container status
     poll_and_print(
         model,
         sc,
         runner,
         get_model_type_config(model.model_type).train_status_descriptions,
-        callback=run.is_ok if run else None,
         wait=wait,
     )
 
@@ -259,7 +223,7 @@ def create(
                 f"\tgretel models get --project {sc.project.name} --model-id {model.model_id} --output .\n"
             )
         )
-    sc.log.info(("Billing estimate" f"\n{json.dumps(model.billing_details, indent=4)}"))
+    sc.log.info((f"Billing estimate\n{json.dumps(model.billing_details, indent=4)}"))
 
     if model.status == Status.COMPLETED:
         sc.log.info(
@@ -305,13 +269,48 @@ def get(sc: SessionContext, project: str, model_id: dict, output: str):
 
 @models.command(help="Search for models of the project.")
 @project_option
-@click.option("--limit", help="Limit the number of projects.", default=100)
-@click.option("--model-name", help="Model name to match on", default="")
+@click.option("--limit", help="Limit the number of projects.", default=100, type=int)
+@click.option("--model-name", help="Model name to match on.", default="")
+@click.option(
+    "--skip",
+    help="Number of models to skip before applying limit.",
+    default=None,
+    type=int,
+)
+@click.option(
+    "--sort-by",
+    help="Direction to sort by. Uses 'asc' if not provided.",
+    type=click.Choice(["asc", "desc"], case_sensitive=False),
+    default=None,
+)
+@click.option(
+    "--sort-field",
+    help="Field to sort on. Uses 'last_modified' if not provided.",
+    default=None,
+)
+@click.option("--status", help="Filter to models with specific status.", default=None)
 @pass_session
-def search(sc: SessionContext, project: str, limit: int, model_name: str):
+def search(
+    sc: SessionContext,
+    project: str,
+    limit: int,
+    model_name: str,
+    skip: Optional[int],
+    sort_by: Optional[str],
+    sort_field: Optional[str],
+    status: Optional[str],
+):
     sc.print(
         data=list(
-            sc.project.search_models(factory=dict, limit=limit, model_name=model_name)
+            sc.project.search_models(
+                factory=dict,
+                limit=limit,
+                model_name=model_name,
+                skip=skip,
+                sort_by=sort_by,
+                sort_field=sort_field,
+                status=status,
+            )
         )
     )
 
