@@ -8,7 +8,7 @@ from typing import Optional
 import click
 import yaml
 
-from gretel_client.cli.common import pass_session, project_option, SessionContext
+from gretel_client.cli.common import SessionContext, pass_session, project_option
 from gretel_client.config import ClientConfig, get_session_config
 from gretel_client.projects.common import WAIT_UNTIL_DONE
 from gretel_client.rest_v1.api.workflows_api import WorkflowsApi
@@ -19,14 +19,13 @@ from gretel_client.rest_v1.models import (
     WorkflowRun,
 )
 from gretel_client.workflows.runner_mode import RunnerMode
-from gretel_client.workflows.status import Status, TERMINAL_STATES
+from gretel_client.workflows.status import TERMINAL_STATES, Status
 
 
 @click.group(
     help="Commands for working with Gretel workflows.",
 )
-def workflows():
-    ...
+def workflows(): ...
 
 
 def _get_workflows_api(*, session: ClientConfig) -> WorkflowsApi:
@@ -76,7 +75,7 @@ def create(
     runner_mode = RunnerMode.from_str(runner_mode)
 
     wfl = CreateWorkflowRequest(
-        config=workflow_config,
+        config_text=yaml.dump(workflow_config, sort_keys=False),
         name=workflow_config["name"],
         project_id=project_id,
         runner_mode=runner_mode,
@@ -90,10 +89,19 @@ def create(
 
 
 @workflows.command(help="List workflows.")
+@click.option(
+    "--limit", metavar="LIMIT", help="Limit the number of returned", default=25
+)
+@click.option(
+    "--skip",
+    metavar="SKIP",
+    help="Skip the specified number of Workflows in the response list",
+    default=0,
+)
 @pass_session
-def list(sc: SessionContext):
+def list(sc: SessionContext, limit: int, skip: int):
     workflow_api = _get_workflows_api(session=sc.session)
-    wfls = workflow_api.get_workflows().workflows
+    wfls = workflow_api.search_workflows(limit=limit, skip=skip).workflows
     if not wfls:
         sc.log.info("No workflows found.")
         return
@@ -136,8 +144,7 @@ def get(sc: SessionContext, id: str):
 @pass_session
 def update(sc: SessionContext, workflow_id: str, config: Path):
     with open(config, encoding="utf-8") as file:
-        # Temporarily load the yaml into a json string and upload that, we can upload raw yaml once the api changes land. -md
-        workflow_config = json.dumps(yaml.safe_load(file))
+        workflow_config = file.read()
     workflow_api = _get_workflows_api(session=sc.session)
     updated_workflow = workflow_api.update_workflow_config(
         workflow_id=workflow_id, body=workflow_config, _content_type="text/yaml"
@@ -153,6 +160,11 @@ def update(sc: SessionContext, workflow_id: str, config: Path):
     help="Wait for workflow run to complete.",
     default=WAIT_UNTIL_DONE,
 )
+@click.option(
+    "--workflow-run-name",
+    metavar="WORKFLOW-RUN-NAME",
+    help="Gretel workflow run name.",
+)
 @click.option(
     "--workflow-id",
     metavar="WORKFLOW-ID",
@@ -160,10 +172,10 @@ def update(sc: SessionContext, workflow_id: str, config: Path):
     required=True,
 )
 @pass_session
-def run(sc: SessionContext, workflow_id: str, wait: int):
+def run(sc: SessionContext, workflow_id: str, workflow_run_name: str, wait: int):
     workflow_api = _get_workflows_api(session=sc.session)
     workflow_run: WorkflowRun = workflow_api.create_workflow_run(
-        CreateWorkflowRunRequest(workflow_id=workflow_id)
+        CreateWorkflowRunRequest(workflow_id=workflow_id, name=workflow_run_name)
     )
     sc.log.info("Workflow run:")
     sc.print(data=workflow_run.to_dict())
