@@ -5,10 +5,15 @@ import logging
 import os
 import platform
 
+from abc import ABC, abstractmethod
+from dataclasses import dataclass, replace
 from enum import Enum
+from functools import cached_property
 from getpass import getpass
+from importlib.metadata import PackageNotFoundError, version
 from pathlib import Path
-from typing import Dict, Optional, Type, TypeVar, Union
+from typing import Optional, Type, TypeVar, Union
+from urllib.parse import quote_plus
 
 import certifi
 
@@ -16,10 +21,13 @@ from urllib3 import HTTPResponse
 from urllib3.exceptions import MaxRetryError
 from urllib3.util import Retry
 
+from gretel_client._api.api_client import ApiClient as V2ApiClient
+from gretel_client._api.configuration import Configuration as V2Configuration
 from gretel_client.rest.api.projects_api import ProjectsApi
 from gretel_client.rest.api.users_api import UsersApi
 from gretel_client.rest.api_client import ApiClient
 from gretel_client.rest.configuration import Configuration
+from gretel_client.rest_v1.api.serverless_api import ServerlessApi
 from gretel_client.rest_v1.api_client import ApiClient as V1ApiClient
 from gretel_client.rest_v1.configuration import Configuration as V1Configuration
 
@@ -48,7 +56,7 @@ GRETEL_PROJECT = "GRETEL_PROJECT"
 GRETEL_RUNNER_MODE = "GRETEL_RUNNER_MODE"
 """Env variable name to set the default runner mode"""
 
-DEFAULT_GRETEL_ENDPOINT = "https://api.gretel.cloud"
+DEFAULT_GRETEL_ENDPOINT = "https://api.gretel.ai"
 """Default gretel endpoint"""
 
 DEFAULT_GRETEL_ARTIFACT_ENDPOINT = "cloud"
@@ -57,7 +65,15 @@ DEFAULT_GRETEL_ARTIFACT_ENDPOINT = "cloud"
 GRETEL_PREVIEW_FEATURES = "GRETEL_PREVIEW_FEATURES"
 """Env variable to manage preview features"""
 
-GRETEL_ENVS = [GRETEL_API_KEY, GRETEL_PROJECT]
+GRETEL_TENANT_NAME = "GRETEL_TENANT_NAME"
+"""Enterprise tenant name"""
+
+GRETEL_TENANT_UNSET = "none"
+"""Value to indicate that a tenant name should be unset"""
+
+GRETEL_ENVS = [GRETEL_API_KEY, GRETEL_PROJECT, GRETEL_TENANT_NAME]
+
+CLIENT_METRICS_HEADER_KEY = "X-Gretel-Client-Metrics"
 
 
 _custom_logger = None
@@ -75,6 +91,42 @@ def configure_custom_logger(logger):
 log = get_logger(__name__)
 
 
+@dataclass
+class Context:
+    client_metrics: dict[str, str]
+    job_provenance: dict[str, str]
+
+    @classmethod
+    def empty(cls) -> Context:
+        return Context(client_metrics={}, job_provenance={})
+
+    def update(
+        self, client_metrics: dict[str, str], job_provenance: dict[str, str]
+    ) -> Context:
+        return replace(
+            self,
+            client_metrics=self.client_metrics | client_metrics,
+            job_provenance=self.job_provenance | job_provenance,
+        )
+
+
+def get_client_version() -> str:
+    try:
+        return version("gretel_client")
+    except PackageNotFoundError:
+        return "undefined"
+
+
+def _metrics_headers(context: Optional[Context] = None) -> dict[str, str]:
+    metadata = {"python_sdk_version": get_client_version()}
+    if context:
+        metadata |= context.client_metrics
+
+    stringified = ";".join((f"{k}={v}" for k, v in metadata.items()))
+
+    return {CLIENT_METRICS_HEADER_KEY: quote_plus(stringified)}
+
+
 class PreviewFeatures(Enum):
     """Manage preview feature configurations"""
 
@@ -82,13 +134,14 @@ class PreviewFeatures(Enum):
     DISABLED = "disabled"
 
 
-class GretelClientConfigurationError(Exception):
-    ...
+class GretelClientConfigurationError(Exception): ...
 
 
-T = TypeVar("T", bound=Type)
-ClientT = TypeVar("ClientT", bound=Union[ApiClient, V1ApiClient])
-ConfigT = TypeVar("ConfigT", bound=Union[Configuration, V1Configuration])
+T = TypeVar("T")
+ClientT = TypeVar("ClientT", bound=Union[ApiClient, V1ApiClient, V2ApiClient])
+ConfigT = TypeVar(
+    "ConfigT", bound=Union[Configuration, V1Configuration, V2Configuration]
+)
 
 
 class RunnerMode(str, Enum):
@@ -188,55 +241,13 @@ class GretelApiRetry(Retry):
         return not was_rate_limited
 
 
-class ClientConfig:
-    """Holds Gretel client configuration details. This can be instantiated from
-    a file or environment.
-    """
-
-    endpoint: str
-    """Gretel API endpoint."""
-
-    artifact_endpoint: str
-    """Artifact endpoint."""
-
-    api_key: Optional[str] = None
-    """Gretel API key."""
-
-    default_project_name: Optional[str] = None
-    """Default Gretel project name."""
-
-    default_runner: str = DEFAULT_RUNNER.value
-    """Default runner"""
-
-    preview_features: str = PreviewFeatures.DISABLED.value
-    """Preview features enabled"""
-
-    def __init__(
-        self,
-        endpoint: Optional[str] = None,
-        artifact_endpoint: Optional[str] = None,
-        api_key: Optional[str] = None,
-        default_project_name: Optional[str] = None,
-        default_runner: Union[str, RunnerMode] = DEFAULT_RUNNER.value,
-        preview_features: str = PreviewFeatures.DISABLED.value,
-    ):
-        self.endpoint = (
-            endpoint or os.getenv(GRETEL_ENDPOINT) or DEFAULT_GRETEL_ENDPOINT
-        )
-        self.artifact_endpoint = (
-            artifact_endpoint
-            or os.getenv(GRETEL_ARTIFACT_ENDPOINT)
-            or DEFAULT_GRETEL_ARTIFACT_ENDPOINT
-        ).removesuffix("/")
-        self.api_key = api_key or os.getenv(GRETEL_API_KEY)
-        self.default_runner = RunnerMode.parse(
-            os.getenv(GRETEL_RUNNER_MODE) or default_runner
-        )
-        self.default_project_name = (
-            default_project_name or os.getenv(GRETEL_PROJECT) or default_project_name
-        )
-        self.preview_features = os.getenv(GRETEL_PREVIEW_FEATURES) or preview_features
-        self._validate()
+class ClientConfig(ABC):
+    def __new__(cls, *args, **kwargs):
+        if cls == ClientConfig:
+            _cls = DefaultClientConfig
+        else:
+            _cls = cls
+        return super().__new__(_cls)
 
     @classmethod
     def from_file(cls, file_path: Path) -> ClientConfig:
@@ -251,10 +262,236 @@ class ClientConfig:
     @classmethod
     def from_dict(cls, source: dict) -> ClientConfig:
         return cls(
-            **{k: v for k, v in source.items() if k in cls.__annotations__.keys()}
+            **{
+                k: v
+                for k, v in source.items()
+                if k in DefaultClientConfig.__annotations__.keys()
+            }
         )
 
+    @property
+    @abstractmethod
+    def context(self) -> Context: ...
+
+    @property
+    @abstractmethod
+    def endpoint(self) -> str: ...
+
+    @property
+    def console_endpoint(self) -> str:
+        if self.stage == "dev":
+            return "https://console-eng.gretel.ai"
+        else:
+            return "https://console.gretel.ai"
+
+    @property
+    @abstractmethod
+    def artifact_endpoint(self) -> str: ...
+
+    @property
+    @abstractmethod
+    def api_key(self) -> Optional[str]: ...
+
+    @property
+    @abstractmethod
+    def default_project_name(self) -> Optional[str]: ...
+
+    @property
+    @abstractmethod
+    def default_runner(self) -> Optional[str]: ...
+
+    @property
+    @abstractmethod
+    def preview_features(self) -> str: ...
+
+    @property
+    @abstractmethod
+    def tenant_name(self) -> Optional[str]: ...
+
+    @cached_property
+    def email(self) -> str:
+        return self.get_api(UsersApi).users_me()["data"]["me"]["email"]
+
+    @property
+    def stage(self) -> str:
+        if (
+            "https://api-dev.gretel" in self.endpoint
+            or ".dev.gretel.cloud" in self.endpoint
+            or ".dev.gretel.ai" in self.endpoint
+        ):
+            return "dev"
+        return "prod"
+
+    @abstractmethod
+    def _get_api_client_generic(
+        self,
+        client_cls: Type[ClientT],
+        config_cls: Type[ConfigT],
+        max_retry_attempts: int = 3,
+        backoff_factor: float = 1,
+        verify_ssl: bool = True,
+        *,
+        default_headers: Optional[dict[str, str]] = None,
+    ) -> ClientT: ...
+
+    def _get_api_client(self, *args, **kwargs) -> ApiClient:
+        return self._get_api_client_generic(ApiClient, Configuration, *args, **kwargs)
+
+    def _get_v1_api_client(self, *args, **kwargs) -> V1ApiClient:
+        return self._get_api_client_generic(
+            V1ApiClient, V1Configuration, *args, **kwargs
+        )
+
+    def set_serverless_api(self) -> bool:
+        serverless: ServerlessApi = self.get_v1_api(ServerlessApi)
+        serverless_tenants_resp = serverless.list_serverless_tenants()
+        if (
+            isinstance(serverless_tenants_resp.tenants, list)
+            and len(serverless_tenants_resp.tenants) > 0
+        ):
+            for tenant in serverless_tenants_resp.tenants:
+                if tenant.name != self.tenant_name:
+                    continue
+                tenant_endpoint = tenant.config.api_endpoint
+                if not tenant_endpoint.startswith("https://"):
+                    tenant_endpoint = f"https://{tenant_endpoint}"
+                if tenant_endpoint != self.endpoint:
+                    print(
+                        "Found a serverless tenant associated with this API key. Updating client configuration to use the tenant API endpoint."
+                    )
+                    self.endpoint = tenant_endpoint
+                return True
+        return False
+
+    def get_api(
+        self,
+        api_interface: Type[T],
+        max_retry_attempts: int = 5,
+        backoff_factor: float = 1,
+        *,
+        default_headers: Optional[dict[str, str]] = None,
+    ) -> T:
+        """Instantiates and configures an api client for a given
+        component interface.
+
+        Args:
+            api_interface: The api interface to instantiate
+            max_retry_attempts: The number of times to retry a failed
+                api request.
+            backoff_factor: A back factor to apply between retry
+                attempts. A base factor of 2 will applied to this value
+                to determine the time between attempts.
+        """
+        return api_interface(
+            self._get_api_client(
+                max_retry_attempts, backoff_factor, default_headers=default_headers
+            )
+        )
+
+    def get_v1_api(
+        self,
+        api_interface: Type[T],
+        max_retry_attempts: int = 5,
+        backoff_factor: float = 1,
+        *,
+        default_headers: Optional[dict[str, str]] = None,
+    ) -> T:
+        return api_interface(
+            self._get_v1_api_client(
+                max_retry_attempts, backoff_factor, default_headers=default_headers
+            )
+        )
+
+    @abstractmethod
+    def update_default_project(self, project_id: str): ...
+
+    @property
+    @abstractmethod
+    def as_dict(self) -> dict: ...
+
+    def __eq__(self, other: ClientConfig) -> bool:
+        return self.as_dict == other.as_dict
+
+    @property
+    def masked(self) -> dict:
+        """Returns a masked representation of the config object."""
+        c = self.as_dict
+        c["api_key"] = self.masked_api_key
+        return c
+
+    @property
+    def masked_api_key(self) -> str:
+        if not self.api_key:
+            return "None"
+
+        return self.api_key[:8] + "****"
+
+    @property
+    def preview_features_enabled(self) -> bool:
+        return self.preview_features == PreviewFeatures.ENABLED.value
+
+
+class DefaultClientConfig(ClientConfig):
+    """Holds Gretel client configuration details. This can be instantiated from
+    a file or environment.
+    """
+
+    endpoint: str = ""
+    """Gretel API endpoint."""
+
+    artifact_endpoint: str = ""
+    """Artifact endpoint."""
+
+    api_key: Optional[str] = None
+    """Gretel API key."""
+
+    default_project_name: Optional[str] = None
+    """Default Gretel project name."""
+
+    default_runner: str = DEFAULT_RUNNER.value
+    """Default runner"""
+
+    preview_features: str = PreviewFeatures.DISABLED.value
+    """Preview features enabled"""
+
+    tenant_name: Optional[str] = None
+    """Enterprise tenant name"""
+
+    def __init__(
+        self,
+        endpoint: Optional[str] = None,
+        console_endpoint: Optional[str] = None,
+        artifact_endpoint: Optional[str] = None,
+        api_key: Optional[str] = None,
+        default_project_name: Optional[str] = None,
+        default_runner: Optional[Union[str, RunnerMode]] = None,
+        preview_features: Optional[str] = None,
+        tenant_name: Optional[str] = None,
+    ):
+        self.endpoint = (
+            endpoint or os.getenv(GRETEL_ENDPOINT) or DEFAULT_GRETEL_ENDPOINT
+        )
+        self.artifact_endpoint = (
+            artifact_endpoint
+            or os.getenv(GRETEL_ARTIFACT_ENDPOINT)
+            or DEFAULT_GRETEL_ARTIFACT_ENDPOINT
+        ).removesuffix("/")
+        self.api_key = api_key or os.getenv(GRETEL_API_KEY)
+        self.default_runner = RunnerMode.parse(
+            default_runner or os.getenv(GRETEL_RUNNER_MODE) or DEFAULT_RUNNER
+        )
+        self.default_project_name = default_project_name or os.getenv(GRETEL_PROJECT)
+        self.preview_features = (
+            preview_features
+            or os.getenv(GRETEL_PREVIEW_FEATURES)
+            or PreviewFeatures.DISABLED.value
+        )
+        self.tenant_name = tenant_name or os.getenv(GRETEL_TENANT_NAME)
+        self._validate()
+
     def _validate(self) -> None:
+        if os.environ.get("GRETEL_ENDPOINT_SKIP_VALIDATION") == "true":
+            return
         if (
             self.artifact_endpoint != DEFAULT_GRETEL_ARTIFACT_ENDPOINT
             and self.default_runner == RunnerMode.CLOUD
@@ -271,6 +508,10 @@ class ClientConfig:
                 "Hybrid runner requires a custom artifact endpoint. Please reconfigure your session."
             )
 
+    @property
+    def context(self) -> Context:
+        return Context.empty()
+
     def _cert_file(self) -> str:
         ssl_cert_file = os.getenv("SSL_CERT_FILE")
         requests_ca_bundle = os.getenv("REQUESTS_CA_BUNDLE")
@@ -306,8 +547,9 @@ class ClientConfig:
         config_cls: Type[ConfigT],
         max_retry_attempts: int = 3,
         backoff_factor: float = 1,
+        verify_ssl: bool = True,
         *,
-        default_headers: Optional[Dict[str, str]] = None,
+        default_headers: Optional[dict[str, str]] = None,
     ) -> ClientT:
         # disable log warnings when the retry kicks in
         logging.getLogger("urllib3.connectionpool").setLevel(logging.ERROR)
@@ -325,6 +567,7 @@ class ClientConfig:
             api_key={"ApiKey": self.api_key},
             ssl_ca_cert=self._cert_file(),
         )
+
         client_kwargs = {}
         if "ApiKey" not in configuration.auth_settings():
             # ApiKey not recognized as authentication method by config,
@@ -338,68 +581,11 @@ class ClientConfig:
             max_retry_attempts=max_retry_attempts,
             backoff_factor=backoff_factor,
         )
+        configuration.verify_ssl = verify_ssl
         client = client_cls(configuration, **client_kwargs)
-        if default_headers:
-            client.default_headers.update(default_headers)
+        client.default_headers.update(_metrics_headers() | (default_headers or {}))
         return client
 
-    def _get_api_client(self, *args, **kwargs) -> ApiClient:
-        return self._get_api_client_generic(ApiClient, Configuration, *args, **kwargs)
-
-    def _get_v1_api_client(self, *args, **kwargs) -> V1ApiClient:
-        return self._get_api_client_generic(
-            V1ApiClient, V1Configuration, *args, **kwargs
-        )
-
-    @property
-    def email(self) -> str:
-        return self.get_api(UsersApi).users_me()["data"]["me"]["email"]
-
-    @property
-    def stage(self) -> str:
-        if "https://api-dev.gretel.cloud" in self.endpoint:
-            return "dev"
-        return "prod"
-
-    def get_api(
-        self,
-        api_interface: Type[T],
-        max_retry_attempts: int = 5,
-        backoff_factor: float = 1,
-        *,
-        default_headers: Optional[Dict[str, str]] = None,
-    ) -> T:
-        """Instantiates and configures an api client for a given
-        component interface.
-
-        Args:
-            api_interface: The api interface to instantiate
-            max_retry_attempts: The number of times to retry a failed
-                api request.
-            backoff_factor: A back factor to apply between retry
-                attempts. A base factor of 2 will applied to this value
-                to determine the time between attempts.
-        """
-        return api_interface(
-            self._get_api_client(
-                max_retry_attempts, backoff_factor, default_headers=default_headers
-            )
-        )
-
-    def get_v1_api(
-        self,
-        api_interface: Type[T],
-        max_retry_attempts: int = 5,
-        backoff_factor: float = 1,
-        *,
-        default_headers: Optional[Dict[str, str]] = None,
-    ) -> T:
-        return api_interface(
-            self._get_v1_api_client(
-                max_retry_attempts, backoff_factor, default_headers=default_headers
-            )
-        )
-
     def _check_project(self, project_name: str = None) -> Optional[str]:
         if not project_name:
             return None
@@ -423,26 +609,170 @@ class ClientConfig:
             if not prop.startswith("_")
         }
 
-    def __eq__(self, other: ClientConfig) -> bool:
-        return self.as_dict == other.as_dict
+    def __repr__(self):
+        # Purposefully don't show the API key
+        return f"""DefaultClientConfig(
+            endpoint="{self.endpoint}",
+            api_key="grtu...",
+            default_project_name="{self.default_project_name}",
+            default_runner="{self.default_runner}",
+            artifact_endpoint={self.artifact_endpoint},
+            tenant_name={self.tenant_name},
+        )"""
+
+
+class DelegatingClientConfig(ClientConfig):
+    _delegate: ClientConfig
+
+    def __init__(self, delegate: ClientConfig):
+        self._delegate = delegate
 
     @property
-    def masked(self) -> dict:
-        """Returns a masked representation of the config object."""
-        c = self.as_dict
-        c["api_key"] = self.masked_api_key
-        return c
+    def context(self) -> Context:
+        return self._delegate.context
 
     @property
-    def masked_api_key(self) -> str:
-        if not self.api_key:
-            return "None"
-
-        return self.api_key[:8] + "****"
+    def endpoint(self) -> str:
+        return self._delegate.endpoint
 
     @property
-    def preview_features_enabled(self) -> bool:
-        return self.preview_features == PreviewFeatures.ENABLED.value
+    def console_endpoint(self) -> str:
+        return self._delegate.console_endpoint
+
+    @property
+    def artifact_endpoint(self) -> str:
+        return self._delegate.artifact_endpoint
+
+    @property
+    def api_key(self) -> Optional[str]:
+        return self._delegate.api_key
+
+    @property
+    def default_project_name(self) -> Optional[str]:
+        return self._delegate.default_project_name
+
+    @property
+    def default_runner(self) -> Optional[str]:
+        return self._delegate.default_runner
+
+    @property
+    def preview_features(self) -> str:
+        return self._delegate.preview_features
+
+    @property
+    def tenant_name(self) -> str:
+        return self._delegate.tenant_name
+
+    def _get_api_client_generic(
+        self,
+        client_cls: Type[ClientT],
+        config_cls: Type[ConfigT],
+        max_retry_attempts: int = 3,
+        backoff_factor: float = 1,
+        verify_ssl: bool = True,
+        *,
+        default_headers: Optional[dict[str, str]] = None,
+    ) -> ClientT:
+        return self._delegate._get_api_client_generic(
+            client_cls=client_cls,
+            config_cls=config_cls,
+            max_retry_attempts=max_retry_attempts,
+            backoff_factor=backoff_factor,
+            verify_ssl=verify_ssl,
+            default_headers=default_headers,
+        )
+
+    def update_default_project(self, project_id: str):
+        self._delegate.update_default_project(project_id)
+
+    @property
+    def as_dict(self) -> dict:
+        return self._delegate.as_dict
+
+
+class TaggedClientConfig(DelegatingClientConfig):
+    _context: Context
+
+    def __init__(self, delegate: ClientConfig, context: Context):
+        super().__init__(delegate)
+        self._context = context
+
+    @property
+    def context(self) -> Context:
+        return self._context
+
+    def _get_api_client_generic(
+        self,
+        client_cls: Type[ClientT],
+        config_cls: Type[ConfigT],
+        max_retry_attempts: int = 3,
+        backoff_factor: float = 1,
+        verify_ssl: bool = True,
+        *,
+        default_headers: Optional[dict[str, str]] = None,
+    ) -> ClientT:
+        all_headers = _metrics_headers(self.context) | (default_headers or {})
+        return super()._get_api_client_generic(
+            client_cls=client_cls,
+            config_cls=config_cls,
+            max_retry_attempts=max_retry_attempts,
+            backoff_factor=backoff_factor,
+            verify_ssl=verify_ssl,
+            default_headers=all_headers,
+        )
+
+    def get_api(
+        self,
+        api_interface: Type[T],
+        max_retry_attempts: int = 5,
+        backoff_factor: float = 1,
+        *,
+        default_headers: Optional[dict[str, str]] = None,
+    ) -> T:
+        all_headers = _metrics_headers(self.context)
+        if default_headers:
+            all_headers |= default_headers
+        return super().get_api(
+            api_interface,
+            max_retry_attempts,
+            backoff_factor,
+            default_headers=all_headers,
+        )
+
+    def get_v1_api(
+        self,
+        api_interface: Type[T],
+        max_retry_attempts: int = 5,
+        backoff_factor: float = 1,
+        *,
+        default_headers: Optional[dict[str, str]] = None,
+    ) -> T:
+        all_headers = _metrics_headers(self.context)
+        if default_headers:
+            all_headers |= default_headers
+        return super().get_v1_api(
+            api_interface,
+            max_retry_attempts,
+            backoff_factor,
+            default_headers=all_headers,
+        )
+
+
+def add_session_context(
+    *,
+    session: Optional[ClientConfig] = None,
+    client_metrics: Optional[dict[str, str]] = None,
+    job_provenance: Optional[dict[str, str]] = None,
+) -> TaggedClientConfig:
+    delegate = session or get_session_config()
+
+    curr_context = delegate.context or Context.empty()
+    new_context = curr_context.update(
+        client_metrics=client_metrics or {},
+        job_provenance=job_provenance or {},
+    )
+
+    return TaggedClientConfig(context=new_context, delegate=delegate)
 
 
 _DEFAULT_CONFIG_PATH = Path().home() / f".{GRETEL}" / "config.json"  # noqa
@@ -566,7 +896,8 @@ def configure_session(
     endpoint: Optional[str] = None,
     artifact_endpoint: Optional[str] = None,
     cache: str = "no",
-    validate: bool = False,
+    validate: bool = True,
+    tenant_name: Optional[str] = None,
     clear: bool = False,
 ):
     """Updates client config for the session
@@ -588,6 +919,7 @@ def configure_session(
             configuration doesn't exist.
         validate: If set to ``True`` this will check that login credentials
             are valid.
+        tenant_name: Specifies the Gretel Enterprise tenant name.
         clear: If set to ``True`` any existing Gretel credentials will be
             removed from the host.
 
@@ -608,6 +940,7 @@ def configure_session(
     default_runner = RunnerMode.parse(default_runner)
 
     artifact_endpoint = artifact_endpoint or config.artifact_endpoint
+    tenant_name = tenant_name or config.tenant_name
 
     api_key = api_key or config.api_key
     if api_key == "prompt":
@@ -625,6 +958,7 @@ def configure_session(
             api_key=api_key,
             default_runner=default_runner,
             default_project_name=config.default_project_name,
+            tenant_name=tenant_name,
         )
 
     if cache == "yes":
@@ -638,8 +972,20 @@ def configure_session(
     if isinstance(config, ClientConfig):
         _session_client_config = config
 
+    if not validate and tenant_name:
+        raise GretelClientConfigurationError(
+            "Cannot run without validation when specifying a tenant name."
+        )
+
+    if tenant_name:
+        if config.set_serverless_api():
+            print(f"Using endpoint {config.endpoint}")
+        else:
+            raise GretelClientConfigurationError(
+                f"Could not find serverless tenant {tenant_name} associated with this API key."
+            )
+
     if validate:
-        print(f"Using endpoint {config.endpoint}")
         try:
             print(f"Logged in as {config.email} \u2705")
         except Exception as ex:
@@ -648,6 +994,26 @@ def configure_session(
             ) from ex
 
 
+def get_data_plane_endpoint(session: Optional[ClientConfig] = None) -> str:
+    if session is None:
+        session = get_session_config()
+
+    if any(
+        session.endpoint.endswith(token)
+        for token in [
+            "sandbox.dev.gretel.ai",
+            "enterprise.dev.gretel.ai",
+            "serverless.dev.gretel.ai",
+        ]
+    ):
+        return session.endpoint
+
+    if "api-dev" in session.endpoint or "api.dev" in session.endpoint:
+        return "https://api.dev.gretel.ai"
+
+    return "https://api.gretel.ai"
+
+
 def _create_mode_opener(mode):
     """Returns an opener to be used with open() that sets the file mode."""
     return lambda path, flags: os.open(path, flags, mode=mode)
