@@ -2,5 +2,4 @@ try:
     from pandas import DataFrame as _DataFrameT
 except ImportError:
 
-    class _DataFrameT:
-        ...  # noqa
+    class _DataFrameT: ...  # noqa
