@@ -1,35 +1,51 @@
+from __future__ import annotations
+
 import logging
 import sys
 import uuid
 
 from pathlib import Path
-from typing import Optional, Union
+from typing import TYPE_CHECKING, Optional, Union
 
-from gretel_client.config import ClientConfig, configure_session, get_session_config
+from gretel_client.config import (
+    ClientConfig,
+    add_session_context,
+    configure_session,
+    get_session_config,
+)
 from gretel_client.dataframe import _DataFrameT
 from gretel_client.factories import GretelFactories
 from gretel_client.gretel.artifact_fetching import (
+    PANDAS_IS_INSTALLED,
     fetch_final_model_config,
     fetch_model_logs,
     fetch_model_report,
     fetch_synthetic_data,
-    PANDAS_IS_INSTALLED,
 )
 from gretel_client.gretel.config_setup import (
     CONFIG_SETUP_DICT,
     create_model_config_from_base,
     extract_model_config_section,
     get_model_docs_url,
+    smart_read_model_config,
 )
 from gretel_client.gretel.exceptions import (
     GretelJobSubmissionError,
     GretelProjectNotSetError,
 )
-from gretel_client.gretel.job_results import GenerateJobResults, TrainJobResults
+from gretel_client.gretel.job_results import (
+    EvaluateResults,
+    GenerateJobResults,
+    TrainJobResults,
+    TransformResults,
+)
 from gretel_client.helpers import poll
-from gretel_client.projects import get_project, Project
+from gretel_client.inference_api.third_party.aws import BedrockAdapter, SagemakerAdapter
+from gretel_client.inference_api.third_party.azure_openai import AzureOpenAIAdapter
+from gretel_client.projects import Project, get_project
+from gretel_client.projects.jobs import Status
 from gretel_client.projects.models import Model
-from gretel_client.rest.exceptions import ApiException
+from gretel_client.rest.exceptions import ApiException, NotFoundException
 from gretel_client.users.users import get_me
 
 try:
@@ -39,14 +55,24 @@ try:
 except ImportError:
     HAS_TUNER = False
 
+if TYPE_CHECKING:
+    from botocore.client import BaseClient
+    from mypy_boto3_sagemaker_runtime import SageMakerRuntimeClient
+    from openai import AzureOpenAI
+
+
 logger = logging.getLogger(__name__)
 logger.propagate = False
 logger.addHandler(logging.StreamHandler(sys.stdout))
 logger.setLevel(logging.INFO)
 
+HIGH_LEVEL_SESSION_METADATA = {"high_level_interface": "1"}
+TRANSFORM_DOCS_URL = "https://docs.gretel.ai/create-synthetic-data/models/transform/v2"
+EVALUATE_DOCS_URL = "https://docs.gretel.ai/optimize-synthetic-data/evaluate"
+
 
 def _convert_to_valid_data_source(
-    data: Optional[Union[str, Path, _DataFrameT]] = None
+    data: Optional[Union[str, Path, _DataFrameT]] = None,
 ) -> Optional[Union[str, _DataFrameT]]:
     """Returns a valid data source (str, DataFrame, or None) for a Gretel job."""
     return str(data) if isinstance(data, Path) else data
@@ -98,19 +124,25 @@ class Gretel:
         project_name: Optional[str] = None,
         project_display_name: Optional[str] = None,
         session: Optional[ClientConfig] = None,
+        skip_configure_session: Optional[bool] = False,
         **session_kwargs,
     ):
         if session is None:
-            if len(session_kwargs) > 0:
+            # Only used for unit tests
+            if not skip_configure_session:
                 configure_session(**session_kwargs)
             session = get_session_config()
         elif len(session_kwargs) > 0:
             raise ValueError("cannot specify session arguments when passing a session")
 
-        self._session = session
-        self._user_id: str = get_me(session=session)["_id"][9:]
+        self._session = add_session_context(
+            session=session, client_metrics=HIGH_LEVEL_SESSION_METADATA
+        )
+        self._user_id: str = get_me(session=self._session)["_id"][9:]
         self._project: Optional[Project] = None
-        self.factories = GretelFactories(session=session)
+        self.factories = GretelFactories(
+            session=self._session, skip_configure_session=skip_configure_session
+        )
 
         if project_name is not None:
             self.set_project(name=project_name, display_name=project_display_name)
@@ -135,7 +167,7 @@ class Gretel:
         if self._project is None:
             logger.info("No project set -> creating a new one...")
             self.set_project(**kwargs)
-        return self._project
+        return self._project  # type: ignore
 
     def set_project(
         self,
@@ -166,8 +198,11 @@ class Gretel:
                 create=True,
                 session=self._session,
             )
-        except ApiException as exception:
-            if "Project name not available" not in exception.body:
+        except (ApiException, NotFoundException) as exception:
+            if (
+                "Project name not available" not in exception.body  # type: ignore
+                and "not found" not in exception.body  # type: ignore
+            ):
                 raise exception
             logger.warning(
                 f"Project name `{name}` is not unique -> "
@@ -288,7 +323,7 @@ class Gretel:
 
             from gretel_client import Gretel
 
-            data_source="https://gretel-public-website.s3-us-west-2.amazonaws.com/datasets/USAdultIncome5k.csv"
+            data_source = "https://gretel-public-website.s3-us-west-2.amazonaws.com/datasets/USAdultIncome5k.csv"
 
             gretel = Gretel(project_name="my-project")
             trained = gretel.submit_train(
@@ -307,8 +342,8 @@ class Gretel:
         data_source = _convert_to_valid_data_source(data_source)
 
         model_type, model_config_section = extract_model_config_section(job_config)
-        model_setup = CONFIG_SETUP_DICT[model_type]
-        model_name = model_setup.model_name.replace("_", "-")
+        model_setup = CONFIG_SETUP_DICT[model_type]  # type: ignore
+        model_name = model_setup.model_name.replace("_", " ")
 
         if data_source is None:
             if not model_setup.data_source_optional:
@@ -324,12 +359,15 @@ class Gretel:
 
         logger.info(
             f"Submitting {model_name.upper()} training job...\n"
-            f"Model Docs:{get_model_docs_url(model_type)}"
+            f"Model Docs: {get_model_docs_url(model_type)}"
         )
 
         model.submit()
 
-        logger.info(f"Console URL: {project_url}/models/{model.model_id}/activity")
+        logger.info(
+            f"Console URL: {project_url}/models/{model.model_id}/activity\n"
+            f"Model ID: {model.model_id}"
+        )
 
         report = None
         logs = None
@@ -342,8 +380,22 @@ class Gretel:
             if (
                 model_setup.report_type is not None
                 and model_config_section.get("data_source") is not None
+                and model.status == Status.COMPLETED
+                and not model_config_section.get("evaluate", {}).get("skip", False)
             ):
-                report = fetch_model_report(model, model_setup.report_type)
+                try:
+                    report = fetch_model_report(model, model_setup.report_type)
+                except Exception as e:
+                    logger.warning(
+                        f"Failed to fetch {model_setup.report_type} report for model {model.model_id}: {e}"
+                    )
+                    report = None
+
+            if model.status != Status.COMPLETED:
+                logger.warning(
+                    "Training didn't complete successfully. Job status was "
+                    f"'{model.status}', details: {model.errors}"
+                )
 
         self._last_model = model
 
@@ -422,12 +474,12 @@ class Gretel:
         )
 
         model_type, _ = extract_model_config_section(model.model_config)
-        model_name = CONFIG_SETUP_DICT[model_type].model_name.replace("_", "-")
+        model_name = CONFIG_SETUP_DICT[model_type].model_name.replace("_", " ")  # type: ignore
         project_url = project.get_console_url()
 
         logger.info(
             f"Submitting {model_name.upper()} generate job...\n"
-            f"Model Docs:{get_model_docs_url(model_type)}\n"
+            f"Model Docs: {get_model_docs_url(model_type)}\n"
             f"Console URL: {project_url}/models/{model.model_id}/data"
         )
 
@@ -438,16 +490,22 @@ class Gretel:
 
         if wait:
             poll(record_handler, verbose=verbose_logging)
-            synthetic_data_link = record_handler.get_artifact_link("data")
-            if fetch_data:
-                if PANDAS_IS_INSTALLED:
-                    synthetic_data = fetch_synthetic_data(record_handler)
-                else:
-                    logger.warning(
-                        "`fetch_data` is True, but pandas is not installed. "
-                        "Only the synthetic data link will be returned. "
-                        "Install pandas by running `pip install pandas`."
-                    )
+            if record_handler.status == Status.COMPLETED:
+                synthetic_data_link = record_handler.get_artifact_link("data")
+                if fetch_data:
+                    if PANDAS_IS_INSTALLED:
+                        synthetic_data = fetch_synthetic_data(record_handler)
+                    else:
+                        logger.warning(
+                            "`fetch_data` is True, but pandas is not installed. "
+                            "Only the synthetic data link will be returned. "
+                            "Install pandas by running `pip install pandas`."
+                        )
+            else:
+                logger.warning(
+                    "Generation didn't complete successfully. Job status was "
+                    f"'{record_handler.status}', details: {record_handler.errors}"
+                )
 
         return GenerateJobResults(
             model=model,
@@ -494,6 +552,7 @@ class Gretel:
         Example::
 
             from gretel_client import Gretel
+
             gretel = Gretel(api_key="prompt")
 
             yaml_config_string = '''
@@ -509,7 +568,7 @@ class Gretel:
                     choices: ["medium", "high"]
             '''
 
-            data_source="https://gretel-public-website.s3-us-west-2.amazonaws.com/datasets/USAdultIncome5k.csv"
+            data_source = "https://gretel-public-website.s3-us-west-2.amazonaws.com/datasets/USAdultIncome5k.csv"
 
             results = gretel.run_tuner(
                 tuner_config=yaml_config_string,
@@ -517,7 +576,7 @@ class Gretel:
                 n_trials=2,
                 params={
                     "batch_size": {"choices": [50, 100]},
-                    "generator_lr": {"log_range": [0.001, 0.01]}
+                    "generator_lr": {"log_range": [0.001, 0.01]},
                 },
                 privacy_filters={"similarity": {"choices": [None, "medium", "high"]}},
             )
@@ -535,12 +594,12 @@ class Gretel:
 
         study = non_default_config_settings.pop("study", None)
 
-        tuner = create_tuner_from_config(
+        tuner = create_tuner_from_config(  # type: ignore
             config=tuner_config, **non_default_config_settings
         )
 
         results = tuner.run(
-            data_source=_convert_to_valid_data_source(data_source),
+            data_source=_convert_to_valid_data_source(data_source),  # type: ignore
             project=self.get_project() if not use_temporary_project else None,
             n_jobs=n_jobs,
             n_trials=n_trials,
@@ -551,6 +610,285 @@ class Gretel:
 
         return results
 
+    def submit_transform(
+        self,
+        config: Union[str, Path, dict],
+        *,
+        data_source: Union[str, Path, _DataFrameT, None],
+        job_label: Optional[str] = None,
+        wait: bool = True,
+        verbose_logging: bool = False,
+    ) -> TransformResults:
+        """Transform a dataset using Gretel Transform v2.
+
+        Args:
+            config: The transform config as a yaml file path, yaml string, or dict.
+            data_source: Training data source as a file path or pandas DataFrame.
+            job_label: Descriptive label to append to job the name.
+            wait: If True, wait for the job to complete before returning.
+            verbose_logging: If True, will print all logs from the job.
+
+        Raises:
+            GretelJobSubmissionError: If any model besides transform v2 is used or an error occurs during submission
+            ModelConfigReadError: If the model config is invalid
+
+        Returns:
+            Transform results dataclass with the transformed_dataframe, transform_logs
+            and as attributes.
+
+        Example::
+
+            from gretel_client import Gretel
+
+            gretel = Gretel(api_key="prompt")
+
+            config = '''schema_version: "1.0"
+
+            name: "transform v2 test model"
+
+            models:
+              - transform_v2:
+                  data_source: "_"
+                  globals:
+                    seed: 123
+                  classify:
+                    enabled: false
+                  steps:
+                    - rows:
+                        update:
+                          - name: age
+                            value: fake(seed=this).bothify(text="##")
+            '''
+
+            data_source = "https://gretel-public-website.s3-us-west-2.amazonaws.com/datasets/USAdultIncome5k.csv"
+
+            transform_result = gretel.submit_transform(
+                config=config,
+                data_source=tabular_data_source,
+                job_label="testing123",
+            )
+
+            # access resulted transform file
+            transformed_data = transform_result.transformed_df
+        """
+        job_config = smart_read_model_config(config, config_name_prefix="transform_v2")
+
+        model_type, _ = extract_model_config_section(job_config)
+        if model_type != "transform_v2":
+            raise GretelJobSubmissionError(
+                "Only transform_v2 is supported for 'submit_transform'"
+            )
+
+        if job_label is not None:
+            job_config["name"] = f"{job_config['name']}-{job_label}"
+
+        data_source = _convert_to_valid_data_source(data_source)
+        project = self.get_project()
+
+        model = project.create_model_obj(
+            model_config=job_config, data_source=data_source
+        )
+        project_url = project.get_console_url()
+        logger.info(
+            f"Submitting transform job...\nTransform Docs: {TRANSFORM_DOCS_URL}"
+        )
+
+        model.submit()
+
+        logger.info(
+            f"Console URL: {project_url}/models/{model.model_id}/activity\n"
+            f"Model ID: {model.model_id}"
+        )
+
+        transform_result = TransformResults(project=project, model=model)
+        if wait:
+            poll(model, verbose=verbose_logging)
+            if model.status != Status.COMPLETED:
+                logger.warning(
+                    "Transform didn't complete successfully. Job status was "
+                    f"'{model.status}', details: {model.errors}"
+                )
+            transform_result.refresh()
+
+        self._last_model = model
+        return transform_result
+
+    def fetch_transform_results(self, model_id: str) -> TransformResults:
+        """Fetch the results object from a Gretel transform job.
+
+        Args:
+            model_id: The Gretel model ID.
+
+        Raises:
+            GretelProjectNotSetError: If a project has not been set.
+
+        Returns:
+            Job results including the model object, report, logs, and final config.
+        """
+        self._assert_project_is_set()
+        model = self.fetch_model(model_id)
+        results = TransformResults(project=self.get_project(), model=model)
+        results.refresh()
+        return results
+
+    def submit_evaluate(
+        self,
+        config: Union[str, Path, dict],
+        *,
+        data_source: Union[str, Path, _DataFrameT, None],
+        ref_data: Union[str, Path, _DataFrameT, None],
+        test_data: Optional[Union[str, Path, _DataFrameT, None]] = None,
+        job_label: Optional[str] = None,
+        wait: bool = True,
+        verbose_logging: bool = False,
+        **non_default_config_settings,
+    ) -> EvaluateResults:
+        """Create an Evaluate Model.
+
+        Args:
+            config: The evaluate config as a yaml file path, yaml string, or dict.
+            data_source: Data source as a file path or pandas DataFrame to train on and produce synthetic data.
+            ref_data: Reference data source as a file path or pandas DataFrame.
+            test_data: Optional test data source as a file path or pandas DataFrame. This file will be used to calculate MIA, for more information about Data Privacy Metrics, please refer to our doc site, https://docs.gretel.ai/optimize-synthetic-data/evaluate/synthetic-data-quality-report.
+            job_label: Descriptive label to append to job the name.
+            wait: If True, wait for the job to complete before returning.
+            verbose_logging: If True, will print all logs from the job.
+
+        Raises:
+            GretelJobSubmissionError: If any model besides evaluate is used or an error occurs during submission
+            ModelConfigReadError: If the model config is invalid
+
+        Returns:
+            Evaluate results dataclass with evaluate_report and evaluate_logs.
+
+        Example::
+
+            from gretel_client import Gretel
+
+            gretel = Gretel(api_key="prompt")
+
+            config = '''schema_version: "1.0"
+
+            name: "evaluate test model"
+
+            models:
+              - evaluate:
+                  data_source: "_"
+            '''
+
+            synth_data_source = "https://gretel-public-website.s3-us-west-2.amazonaws.com/datasets/USAdultIncome5kGenerated.csv"
+            train_ref_data = "https://gretel-public-website.s3-us-west-2.amazonaws.com/datasets/USAdultIncome4k.csv"
+            test_data = "https://gretel-public-website.s3.us-west-2.amazonaws.com/datasets/USAdultIncome1k.csv"
+            evaluate_result = gretel.submit_evaluate(
+                config=config,
+                data_source=synth_data_source,
+                ref_data=train_ref_data,
+                test_data=test_data,
+                job_label="testing123",
+            )
+
+            # access resulted evaluate report
+            evaluate_report = evaluate_result.evaluate_report
+        """
+        job_config = smart_read_model_config(config, config_name_prefix="evaluate")
+
+        model_type, model_config_section = extract_model_config_section(job_config)
+
+        if model_type != "evaluate":
+            raise GretelJobSubmissionError(
+                "Only `evaluate` is supported for 'submit_evaluate'"
+            )
+
+        if non_default_config_settings:
+            model_config_section.update(non_default_config_settings)
+
+        if job_label is not None:
+            job_config["name"] = f"{job_config['name']}-{job_label}"
+
+        data_source = _convert_to_valid_data_source(data_source)
+        ref_data = _convert_to_valid_data_source(ref_data)
+        if test_data is not None:
+            test_data = _convert_to_valid_data_source(test_data)
+            ref_data = [ref_data, test_data]
+
+        project = self.get_project()
+
+        model = project.create_model_obj(
+            model_config=job_config, data_source=data_source, ref_data=ref_data
+        )
+        project_url = project.get_console_url()
+        logger.info(f"Submitting evaluate job...\nEvaluate Docs: {EVALUATE_DOCS_URL}")
+
+        model.submit()
+
+        logger.info(
+            f"Console URL: {project_url}/models/{model.model_id}/activity\n"
+            f"Model ID: {model.model_id}"
+        )
+
+        evaluate_result = EvaluateResults(project=project, model=model)
+        if wait:
+            poll(model, verbose=verbose_logging)
+            if model.status != Status.COMPLETED:
+                logger.warning(
+                    "Evaluate didn't complete successfully. Job status was "
+                    f"'{model.status}', details: {model.errors}"
+                )
+            evaluate_result.refresh()
+
+        self._last_model = model
+        return evaluate_result
+
     def __repr__(self):
         name = self._project.name if self._project else None
         return f"{self.__class__.__name__}(project_name={name})"
+
+    @staticmethod
+    def create_navigator_azure_oai_adapter(
+        open_ai_client: AzureOpenAI,
+    ) -> AzureOpenAIAdapter:
+        """
+        Create an adapter for using Gretel's Navigator Tabular API with Azure OpenAI endpoints.
+
+        Args:
+            open_ai_client: An instance of AzureOpenAI from the `openai` package.
+        """
+        if not PANDAS_IS_INSTALLED:
+            raise ImportError(
+                "This adapter requires pandas. To install, run `pip install pandas`"
+            )
+        return AzureOpenAIAdapter(open_ai_client)
+
+    @staticmethod
+    def create_navigator_sagemaker_adapter(
+        sagemaker_client: SageMakerRuntimeClient, endpoint_name: str
+    ) -> SagemakerAdapter:
+        """
+        Create an adapter for using Gretel's Navigator Tabular API with AWS SageMaker endpoints.
+
+        Args:
+            sagemaker_client: An instance of SageMakerRuntimeClient from the `mypy_boto3_sagemaker_runtime` package.
+            endpoint_name: The name of the SageMaker endpoint.
+        """
+        if not PANDAS_IS_INSTALLED:
+            raise ImportError(
+                "This adapter requires pandas. To install, run `pip install pandas`"
+            )
+        return SagemakerAdapter(sagemaker_client, endpoint_name)
+
+    @staticmethod
+    def create_navigator_bedrock_adapter(
+        client: BaseClient, endpoint_arn: str
+    ) -> BedrockAdapter:
+        """
+        Create an adapter for using Gretel's Navigator Tabular API with Bedrock endpoints.
+
+        Args:
+            client: An instance of the bedrock client from the `boto3` package.
+            endpoint_arn: The ARN of the Bedrock (Sagemaker) endpoint.
+        """
+        if not PANDAS_IS_INSTALLED:
+            raise ImportError(
+                "This adapter requires pandas. To install, run `pip install pandas`"
+            )
+        return BedrockAdapter(client, endpoint_arn)
