@@ -8,9 +8,10 @@ from typing import Any, Iterator, List, Optional, Union
 
 from tqdm import tqdm
 
+from gretel_client import analysis_utils
 from gretel_client.dataframe import _DataFrameT
 from gretel_client.gretel.artifact_fetching import PANDAS_IS_INSTALLED
-from gretel_client.gretel.config_setup import TabLLMDefaultParams
+from gretel_client.gretel.config_setup import NavigatorDefaultParams
 from gretel_client.inference_api.base import BaseInferenceAPI, GretelInferenceAPIError
 
 if PANDAS_IS_INSTALLED:
@@ -22,23 +23,23 @@ logger.addHandler(logging.StreamHandler(sys.stdout))
 logger.setLevel(logging.INFO)
 
 STREAM_SLEEP_TIME = 0.5
-MAX_ROWS_PER_STREAM = 50
+MAX_ROWS_PER_STREAM = 100
 REQUEST_TIMEOUT_SEC = 60
 TABULAR_API_PATH = "/v1/inference/tabular/"
-TABLLM_DEFAULT_MODEL = "gretelai/tabular-v0"
-
+PROGRESS_BAR_FORMAT = "{l_bar}{bar}| {n_fmt}/{total_fmt} [{elapsed},{rate_noinv_fmt}]"
 
 StreamReturnType = Union[
     Iterator[dict[str, Any]], _DataFrameT, List[dict[str, Any]], dict[str, Any]
 ]
 
 
-class TabularLLMInferenceAPI(BaseInferenceAPI):
-    """Inference API for real-time data generation with Gretel's tabular LLM.
+class TabularInferenceAPI(BaseInferenceAPI):
+    """Inference API for real-time data generation with Gretel Navigator.
 
     Args:
         backend_model (str, optional): The model that is used under the hood.
-            See the `backend_model_list` property for a list of available models.
+            If None, the latest default model will be used. See the
+            `backend_model_list` property for a list of available models.
         **session_kwargs: kwargs for your Gretel session.
 
     Raises:
@@ -56,14 +57,20 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
     a new request will automatically be made to continue record generation.
     """
 
-    def __init__(self, backend_model: str = "gretelai/tabular-v0", **session_kwargs):
-        super().__init__(**session_kwargs)
-        self.backend_model = backend_model
+    max_retry_count: int = 3
+    """
+    How many errors or timeouts should be tolerated before the generation
+    process raises a user-facing error.
+    """
 
     @property
     def api_path(self) -> str:
         return TABULAR_API_PATH
 
+    @property
+    def model_type(self) -> str:
+        return "tabular"
+
     @property
     def stream_api_path(self) -> str:
         return self.api_path + "stream"
@@ -76,34 +83,24 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
     def generate_api_path(self) -> str:
         return self.api_path + "generate"
 
-    @property
-    def backend_model(self) -> str:
-        return self._backend_model
-
-    @backend_model.setter
-    def backend_model(self, backend_model: str) -> None:
-        if backend_model not in self.backend_model_list:
-            raise GretelInferenceAPIError(
-                f"Model {backend_model} is not a valid backend model. "
-                f"Valid models are: {self.backend_model_list}"
-            )
-        self._backend_model = backend_model
-        logger.info("Backend model: %s", backend_model)
-        self._reset_stream()
-
-    @property
-    def backend_model_list(self) -> List[str]:
-        """Returns list of available tabular backend models."""
-        return [
-            m["model_id"]
-            for m in self._available_backend_models
-            if m["model_type"].upper() == "TABULAR"
-        ]
-
     @property
     def name(self) -> str:
         """Returns display name for this inference api."""
-        return "Gretel Tabular LLM"
+        return "Navigator Tabular"
+
+    def display_dataframe_in_notebook(
+        self, dataframe: _DataFrameT, settings: Optional[dict] = None
+    ) -> None:
+        """Display pandas DataFrame in notebook with better settings for readability.
+
+        This function is intended to be used in a Jupyter notebook.
+
+        Args:
+            dataframe: The pandas DataFrame to display.
+            settings: Optional properties to set on the DataFrame's style.
+                If None, default settings with text wrapping are used.
+        """
+        analysis_utils.display_dataframe_in_notebook(dataframe, settings)
 
     def _reset_stream(self) -> None:
         """Reset the stream state."""
@@ -111,6 +108,7 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
         self._last_stream_read = None
         self._next_iter = None
         self._generated_count = 0
+        self._set_response_metadata({})
 
     def _create_stream_if_needed(
         self,
@@ -183,7 +181,13 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
         self._reset_stream()
         done_generation = False
 
-        while self._generated_count < num_records:
+        attempt_count = 0
+        response_metadata: dict = {}
+
+        # Keep going until we've given the user all their records and the most recent
+        # stream has been closed. We need to ensure the last stream is closed because
+        # that's when we receive additional metadata (model_ids, billing, etc).
+        while self._generated_count < num_records or self._curr_stream_id:
             if history_buffer:
                 this_ref_data["sample_data"] = {
                     "table_headers": list(history_buffer[0].keys()),
@@ -216,20 +220,37 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
             for record in data_list:
                 if record["data_type"] == "logger.info":
                     logger.debug("%s: %s", self.__class__.__name__, record["data"])
+                elif record["data_type"] == "ResponseMetadata":
+                    response_metadata = _combine_response_metadata(
+                        response_metadata, json.loads(record["data"])
+                    )
+                    self._set_response_metadata(response_metadata)
                 elif record["data_type"] == "TabularResponse":
                     row_data = record["data"]["table_data"]
                     for row in row_data:
                         self._generated_count += 1
                         self._last_stream_read = time.time()
+                        if self._generated_count > num_records:
+                            # If we've already sent the user back all of their requested results,
+                            # don't bother returning more data than they expect.
+                            continue
                         if sample_buffer_size > 0:
                             history_buffer.append(row)
                             history_buffer = history_buffer[-sample_buffer_size:]
                         yield row
-                    if self._generated_count >= num_records:
-                        done_generation = True
-                        break
                 elif record["data_type"] == "logger.error":
-                    raise GretelInferenceAPIError(record["data"])
+                    attempt_count += 1
+                    err_string = record["data"]
+                    if attempt_count >= self.max_retry_count:
+                        raise GretelInferenceAPIError(
+                            f"Failed to generate data after {attempt_count} attempts, error from API: {err_string}"
+                        )
+                    else:
+                        logger.warning(
+                            "Received error during generation: '%s'. Retrying.",
+                            err_string,
+                        )
+                        self._curr_stream_id = None
 
             # The stream is exhausted when the state is closed and there's no more data.
             if resp.get("stream_state", {}).get("status") == "closed" and not data_list:
@@ -249,6 +270,11 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
                     self._curr_stream_id,
                     self.request_timeout_sec,
                 )
+                attempt_count += 1
+                if attempt_count >= self.max_retry_count:
+                    raise GretelInferenceAPIError(
+                        f"Failed to generate data after {attempt_count} attempts."
+                    )
                 self._curr_stream_id = None
 
             time.sleep(STREAM_SLEEP_TIME)
@@ -280,7 +306,11 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
 
         generated_records = []
         with tqdm(
-            total=num_records, desc=pbar_desc, disable=disable_pbar, initial=1
+            total=num_records,
+            desc=pbar_desc,
+            disable=disable_pbar,
+            unit=" records",
+            bar_format=PROGRESS_BAR_FORMAT,
         ) as pbar:
             for record in stream_iterator:
                 generated_records.append(record)
@@ -302,9 +332,9 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
         *,
         seed_data: Union[_DataFrameT, List[dict[str, Any]]],
         chunk_size: int = 25,
-        temperature: float = TabLLMDefaultParams.temperature,
-        top_k: int = TabLLMDefaultParams.top_k,
-        top_p: float = TabLLMDefaultParams.top_p,
+        temperature: float = NavigatorDefaultParams.temperature,
+        top_k: int = NavigatorDefaultParams.top_k,
+        top_p: float = NavigatorDefaultParams.top_p,
         stream: bool = False,
         as_dataframe: bool = True,
         disable_progress_bar: bool = False,
@@ -336,7 +366,7 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
 
         Example::
 
-            from gretel_client.inference_api.tabular import TabularLLMInferenceAPI
+            from gretel_client.inference_api.tabular import TabularInferenceAPI
 
             # Example seed data if using a list of dicts.
             # You can also use a pandas DataFrame.
@@ -352,28 +382,16 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
                     "last_name": "Simpson",
                     "favorite_band": "The Beatles",
                     "favorite_tv_show": "Friends",
-                }
+                },
             ]
 
             prompt = "Please add a column with the character's favorite food."
 
-            tabllm = TabularLLMInferenceAPI(api_key="prompt")
+            tabular = TabularInferenceAPI(api_key="prompt")
 
-            df = tabllm.edit(prompt=prompt, seed_data=seed_data)
+            df = tabular.edit(prompt=prompt, seed_data=seed_data)
         """
-        if isinstance(seed_data, list) and isinstance(seed_data[0], dict):
-            table_headers = list(seed_data[0].keys())
-            table_data = seed_data
-        elif PANDAS_IS_INSTALLED and isinstance(seed_data, pd.DataFrame):
-            table_headers = list(seed_data.columns)
-            # By using `to_json()` we serialize out any Pandas specific data types
-            # to scalar values first
-            table_data = json.loads(seed_data.to_json(orient="records"))
-        else:
-            raise GretelInferenceAPIError(
-                "Seed data must be a `pandas` DataFrame or a list of dicts."
-            )
-
+        table_headers, table_data = _data_input_to_api_data(seed_data, "Seed data")
         total_record_count = len(table_data)
 
         # Replace our list of records to edit with a list of lists
@@ -427,9 +445,10 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
         prompt: str,
         *,
         num_records: int,
-        temperature: float = TabLLMDefaultParams.temperature,
-        top_k: int = TabLLMDefaultParams.top_k,
-        top_p: float = TabLLMDefaultParams.top_p,
+        temperature: float = NavigatorDefaultParams.temperature,
+        top_k: int = NavigatorDefaultParams.top_k,
+        top_p: float = NavigatorDefaultParams.top_p,
+        sample_data: Optional[Union[_DataFrameT, List[dict[str, Any]]]] = None,
         stream: bool = False,
         as_dataframe: bool = True,
         disable_progress_bar: bool = False,
@@ -450,14 +469,19 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
         LLM will use that as context for record generation. This is useful
         for keeping continuity in fields between requests (such as monotonic values, etc).
 
-
-
         Args:
             prompt: The prompt for generating synthetic tabular data.
             num_records: The number of records to generate.
             temperature: Sampling temperature. Higher values make output more random.
             top_k: Number of highest probability tokens to keep for top-k filtering.
             top_p: The cumulative probability cutoff for sampling tokens.
+            sample_data: The sample data to guide the initial generation process.
+                Things to keep in mind when using this parameter:
+                - The generated data will use exact column names from "sample_data".
+                - It's important for the prompt and "sample_data" match.
+                  For example, they should refer to the same columns.
+                - Use sample data to provide examples of the data you want generated,
+                  E.g. if you need specific data formats.
             stream: If True, stream the generated data.
             as_dataframe: If True, return the data as a pandas DataFrame. This
                 parameter is ignored if `stream` is True.
@@ -472,7 +496,7 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
 
         Example::
 
-            from gretel_client.inference_api.tabular import TabularLLMInferenceAPI
+            from gretel_client.inference_api.tabular import TabularInferenceAPI
 
             prompt = (
                 "Generate positive and negative reviews for the following products: "
@@ -480,10 +504,36 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
                 "Include columns for the product name, number of stars (1-5), review, and customer id."
             )
 
-            tabllm = TabularLLMInferenceAPI(api_key="prompt")
+            tabular = TabularInferenceAPI(api_key="prompt")
 
-            df = tabllm.generate(prompt=prompt, num_records=10)
+            df = tabular.generate(prompt=prompt, num_records=10)
+
+            # Another example with sample data
+            sample_data = [
+                {
+                    "review_date": "2021-01-01",
+                    "product_name": "red travel mug",
+                    "stars": 5,
+                    "review": "I love this mug!",
+                    "customer_id": 123,
+                }
+            ]
+            df = tabular.generate(
+                prompt=prompt, num_records=10, sample_data=sample_data
+            )
         """
+        ref_data = {}
+        if sample_data:
+            table_headers, table_data = _data_input_to_api_data(
+                sample_data, "Sample data"
+            )
+            ref_data = {
+                "sample_data": {
+                    "table_headers": table_headers,
+                    "table_data": table_data,
+                }
+            }
+
         stream_iterator = self._stream(
             prompt=prompt,
             num_records=num_records,
@@ -493,6 +543,7 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
                 "top_p": top_p,
             },
             sample_buffer_size=sample_buffer_size,
+            ref_data=ref_data,
         )
 
         return self._get_stream_results(
@@ -503,3 +554,44 @@ class TabularLLMInferenceAPI(BaseInferenceAPI):
             pbar_desc="Generating records",
             disable_pbar=disable_progress_bar,
         )
+
+
+def _data_input_to_api_data(
+    data: Union[_DataFrameT, List[dict[str, Any]]], data_type: str
+) -> tuple[list[str], list[dict[str, Any]]]:
+    """
+    Converts data input from the high level SDK to the format that our API expects:
+    - table_headers: contains list of columns
+    - table_data: contains list of records
+
+    Raises:
+        GretelInferenceAPIError: If the data input is invalid.
+    """
+    if isinstance(data, list) and isinstance(data[0], dict):
+        table_headers = list(data[0].keys())
+        table_data = data
+    elif PANDAS_IS_INSTALLED and isinstance(data, pd.DataFrame):
+        table_headers = list(data.columns)
+        # By using `to_json()` we serialize out any Pandas specific data types
+        # to scalar values first
+        table_data = json.loads(data.to_json(orient="records"))
+    else:
+        raise GretelInferenceAPIError(
+            f"{data_type} must be a `pandas` DataFrame or a list of dicts."
+        )
+
+    return table_headers, table_data
+
+
+def _combine_response_metadata(
+    response_metadata: dict, new_response_data: dict
+) -> dict:
+    if not response_metadata:
+        return new_response_data
+
+    for k, v in new_response_data["usage"].items():
+        if not isinstance(v, int):
+            continue
+        response_metadata["usage"][k] += v
+
+    return response_metadata
