@@ -7,21 +7,22 @@ import tempfile
 import uuid
 
 from contextlib import contextmanager
+from functools import cached_property
 from pathlib import Path
-from typing import Any, BinaryIO, Dict, IO, List, Optional, Protocol, Tuple, Type, Union
+from typing import IO, Any, BinaryIO, Dict, List, Optional, Protocol, Tuple, Type, Union
 from urllib.parse import urlparse
 
 import requests
 import smart_open
+import urllib3
 
-from backports.cached_property import cached_property
 from tenacity import retry, retry_if_exception_type, stop_after_attempt, wait_fixed
 
 import gretel_client.projects.common as common
 
-from gretel_client.config import ClientConfig, DEFAULT_GRETEL_ARTIFACT_ENDPOINT
+from gretel_client.config import DEFAULT_GRETEL_ARTIFACT_ENDPOINT, ClientConfig
 from gretel_client.dataframe import _DataFrameT
-from gretel_client.projects.common import f, ModelArtifact, ModelRunArtifact, Pathlike
+from gretel_client.projects.common import ModelArtifact, ModelRunArtifact, Pathlike, f
 from gretel_client.rest.api.projects_api import ProjectsApi
 from gretel_client.rest.exceptions import NotFoundException
 from gretel_client.rest.model.artifact import Artifact
@@ -37,11 +38,23 @@ except ImportError:  # pragma: no cover
 
 HYBRID_ARTIFACT_ENDPOINT_PREFIXES = ["azure://", "gs://", "s3://"]
 
+GITHUB_RAW = "raw.githubusercontent.com"
+GRETEL_GITHUB_BLUEPRINT_REPO = "/gretelai/gretel-blueprints"
+
+GRETEL_SOURCES = {
+    "gretel-datasets.s3.amazonaws.com",
+    "gretel-public-website.s3.us-west-2.amazonaws.com",
+    "blueprints.gretel.cloud",
+    "blueprints-dev.gretel.cloud",
+}
+
 
 def _get_azure_blob_srv_client() -> Optional[BlobServiceClient]:
     for env_var_name in ("AZURE_STORAGE_ACCOUNT_NAME", "OAUTH_STORAGE_ACCOUNT_NAME"):
         if (storage_account := os.getenv(env_var_name)) is not None:
-            oauth_url = f"https://{storage_account}.blob.core.windows.net"
+            domain = os.getenv("AZURE_STORAGE_DOMAIN") or "windows.net"
+            normalized_domain = domain.removeprefix(".")
+            oauth_url = f"https://{storage_account}.blob.core.{normalized_domain}"
             return BlobServiceClient(
                 account_url=oauth_url, credential=DefaultAzureCredential()
             )
@@ -100,26 +113,22 @@ class ManifestPendingException(Exception):
 
 class _Project(Protocol):
     @property
-    def project_id(self) -> str:
-        ...
+    def project_id(self) -> str: ...
 
     @property
-    def name(self) -> str:
-        ...
+    def name(self) -> str: ...
 
     @property
-    def projects_api(self) -> ProjectsApi:
-        ...
+    def projects_api(self) -> ProjectsApi: ...
 
     @property
-    def client_config(self) -> ClientConfig:
-        ...
+    def client_config(self) -> ClientConfig: ...
 
 
 def cloud_handler(project: _Project) -> CloudArtifactsHandler:
     return CloudArtifactsHandler(
         projects_api=project.projects_api,
-        project_id=project.project_id,
+        project_guid=project.project_guid,
         project_name=project.name,
     )
 
@@ -140,45 +149,36 @@ class ArtifactsHandler(Protocol):
     def validate_data_source(
         self,
         artifact_path: Pathlike,
-    ) -> bool:
-        ...
+    ) -> bool: ...
 
     def upload_project_artifact(
         self,
         artifact_path: Union[Path, str, _DataFrameT],
-    ) -> str:
-        ...
+    ) -> str: ...
 
-    def delete_project_artifact(self, key: str) -> None:
-        ...
+    def delete_project_artifact(self, key: str) -> None: ...
 
-    def list_project_artifacts(self) -> List[dict]:
-        ...
+    def list_project_artifacts(self) -> List[dict]: ...
 
-    def get_project_artifact_link(self, key: str) -> str:
-        ...
+    def get_project_artifact_link(self, key: str) -> str: ...
 
-    def get_project_artifact_handle(self, key: str) -> BinaryIO:
-        ...
+    def get_project_artifact_handle(self, key: str) -> BinaryIO: ...
 
     def get_project_artifact_manifest(
         self,
         key: str,
         retry_on_not_found: bool = True,
         retry_on_pending: bool = True,
-    ) -> Dict[str, Any]:
-        ...
+    ) -> Dict[str, Any]: ...
 
-    def get_model_artifact_link(self, model_id: str, artifact_type: str) -> str:
-        ...
+    def get_model_artifact_link(self, model_id: str, artifact_type: str) -> str: ...
 
     def get_record_handler_artifact_link(
         self,
         model_id: str,
         record_handler_id: str,
         artifact_type: str,
-    ) -> str:
-        ...
+    ) -> str: ...
 
     def download(
         self,
@@ -186,14 +186,13 @@ class ArtifactsHandler(Protocol):
         output_path: Path,
         artifact_type: str,
         log: logging.Logger,
-    ) -> None:
-        ...
+    ) -> None: ...
 
 
 class CloudArtifactsHandler:
-    def __init__(self, projects_api: ProjectsApi, project_id: str, project_name: str):
+    def __init__(self, projects_api: ProjectsApi, project_guid: str, project_name: str):
         self.projects_api = projects_api
-        self.project_id = project_id
+        self.project_guid = project_guid
         self.project_name = project_name
 
     def validate_data_source(
@@ -211,9 +210,9 @@ class CloudArtifactsHandler:
 
         with _get_artifact_path_and_file_name(artifact_path) as art_path_and_file:
             artifact_path, file_name = art_path_and_file
-            with smart_open.open(artifact_path, "rb", ignore_ext=True) as src:
+            with smart_open.open(artifact_path, "rb", compression="disable") as src:
                 art_resp = self.projects_api.create_artifact(
-                    project_id=self.project_name, artifact=Artifact(filename=file_name)
+                    project_id=self.project_guid, artifact=Artifact(filename=file_name)
                 )
                 artifact_key = art_resp[f.DATA][f.KEY]
                 url = art_resp[f.DATA][f.URL]
@@ -224,21 +223,40 @@ class CloudArtifactsHandler:
     def _does_not_require_upload(
         self, artifact_path: Union[Path, str, _DataFrameT]
     ) -> bool:
-        return isinstance(artifact_path, str) and artifact_path.startswith("gretel_")
+        return isinstance(artifact_path, str) and (
+            self._is_gretel_project_artifact_key(artifact_path)
+            or self._is_public_gretel_artifact(artifact_path)
+        )
+
+    def _is_gretel_project_artifact_key(self, artifact_path: str) -> bool:
+        return artifact_path.startswith("gretel_")
+
+    def _is_public_gretel_artifact(self, artifact_path: str) -> bool:
+        try:
+            parsed_url: urllib3.util.Url = urllib3.util.parse_url(artifact_path)
+            if parsed_url.scheme is not None:
+                if (
+                    parsed_url.hostname == GITHUB_RAW
+                    and parsed_url.path.startswith(GRETEL_GITHUB_BLUEPRINT_REPO)
+                ) or parsed_url.hostname in GRETEL_SOURCES:
+                    return True
+        except Exception:
+            pass
+        return False
 
     def delete_project_artifact(self, key: str) -> None:
-        return self.projects_api.delete_artifact(project_id=self.project_name, key=key)
+        return self.projects_api.delete_artifact(project_id=self.project_guid, key=key)
 
     def list_project_artifacts(self) -> List[dict]:
         return (
-            self.projects_api.get_artifacts(project_id=self.project_name)
+            self.projects_api.get_artifacts(project_id=self.project_guid)
             .get(f.DATA)
             .get(f.ARTIFACTS)
         )
 
     def get_project_artifact_link(self, key: str) -> str:
         resp = self.projects_api.download_artifact(
-            project_id=self.project_name, key=key
+            project_id=self.project_guid, key=key
         )
         return resp[f.DATA][f.DATA][f.URL]
 
@@ -276,7 +294,7 @@ class CloudArtifactsHandler:
         resp = None
         try:
             resp = self.projects_api.get_artifact_manifest(
-                project_id=self.project_name, key=key
+                project_id=self.project_guid, key=key
             )
         except NotFoundException:
             raise ManifestNotFoundException()
@@ -288,7 +306,7 @@ class CloudArtifactsHandler:
 
     def get_model_artifact_link(self, model_id: str, artifact_type: str) -> str:
         art_resp = self.projects_api.get_model_artifact(
-            project_id=self.project_name, model_id=model_id, type=artifact_type
+            project_id=self.project_guid, model_id=model_id, type=artifact_type
         )
         return art_resp[f.DATA][f.URL]
 
@@ -299,7 +317,7 @@ class CloudArtifactsHandler:
         artifact_type: str,
     ) -> str:
         resp = self.projects_api.get_record_handler_artifact(
-            project_id=self.project_name,
+            project_id=self.project_guid,
             model_id=model_id,
             record_handler_id=record_handler_id,
             type=artifact_type,
@@ -350,11 +368,14 @@ class HybridArtifactsHandler:
             data_source_file_name = f"gretel_{uuid.uuid4().hex}_{file_name}"
             target_out = f"{self.data_sources_dir}/{data_source_file_name}"
 
-            with open_artifact(
-                artifact_path,
-                "rb",
-                ignore_ext=True,
-            ) as in_stream, open_artifact(target_out, "wb") as out_stream:
+            with (
+                open_artifact(
+                    artifact_path,
+                    "rb",
+                    compression="disable",
+                ) as in_stream,
+                open_artifact(target_out, "wb") as out_stream,
+            ):
                 shutil.copyfileobj(in_stream, out_stream)
 
             return target_out
@@ -501,9 +522,10 @@ def _download(
 ) -> None:
     target_out = output_path / Path(urlparse(download_link).path).name
     try:
-        with open_artifact(download_link, "rb", ignore_ext=True) as src, open_artifact(
-            target_out, "wb", ignore_ext=True
-        ) as dest:
+        with (
+            open_artifact(download_link, "rb", compression="disable") as src,
+            open_artifact(target_out, "wb", compression="disable") as dest,
+        ):
             shutil.copyfileobj(src, dest)
         log.info(f"Downloaded '{artifact_type}' artifact")
     except Exception:
@@ -522,7 +544,7 @@ def _download(
 
 @contextmanager
 def _get_artifact_path_and_file_name(
-    artifact_path: Union[Path, str, _DataFrameT]
+    artifact_path: Union[Path, str, _DataFrameT],
 ) -> Tuple[str, str]:
     if isinstance(artifact_path, _DataFrameT):
         with tempfile.NamedTemporaryFile(delete=False) as tmp_file:
