@@ -4,12 +4,13 @@ import base64
 import json
 import time
 
-from abc import ABC, abstractmethod, abstractproperty
+from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from dataclasses import dataclass, field
 from enum import Enum
 from pathlib import Path
 from typing import (
+    TYPE_CHECKING,
     BinaryIO,
     Callable,
     Iterator,
@@ -17,7 +18,6 @@ from typing import (
     Optional,
     Tuple,
     Type,
-    TYPE_CHECKING,
     Union,
 )
 
@@ -26,18 +26,25 @@ import smart_open
 import gretel_client.rest.exceptions
 
 from gretel_client.cli.utils.parser_utils import RefData
-from gretel_client.config import ClientConfig, get_logger, RunnerMode
+from gretel_client.config import ClientConfig, RunnerMode, get_logger
 from gretel_client.dataframe import _DataFrameT
 from gretel_client.models.config import get_model_type_config
 from gretel_client.projects.artifact_handlers import (
     ArtifactsHandler,
     CloudArtifactsHandler,
-    get_transport_params,
     HybridArtifactsHandler,
+    get_transport_params,
+)
+from gretel_client.projects.common import WAIT_UNTIL_DONE, ModelArtifact, f
+from gretel_client.projects.exceptions import (
+    CreditExhaustException,
+    DiscontinuedModelException,
+    GretelJobNotFound,
+    MaxConcurrentJobsException,
+    WaitTimeExceeded,
 )
-from gretel_client.projects.common import f, ModelArtifact, WAIT_UNTIL_DONE
-from gretel_client.projects.exceptions import GretelJobNotFound, WaitTimeExceeded
 from gretel_client.rest.api.projects_api import ProjectsApi
+from gretel_client.rest.exceptions import ApiException
 
 if TYPE_CHECKING:
     from gretel_client.projects import Project
@@ -195,36 +202,31 @@ class Job(ABC):
         return self._submit(runner_mode=runner_mode, dry_run=dry_run)
 
     @abstractmethod
-    def _submit(self, runner_mode: RunnerMode, **kwargs) -> Job:
-        ...
+    def _submit(self, runner_mode: RunnerMode, **kwargs) -> Job: ...
 
-    @abstractproperty
-    def model_type(self) -> str:
-        ...
+    @property
+    @abstractmethod
+    def model_type(self) -> str: ...
 
     @abstractmethod
-    def _do_get_job_details(self, extra_expand: Optional[list[str]] = None) -> dict:
-        ...
+    def _do_get_job_details(self, extra_expand: Optional[list[str]] = None) -> dict: ...
 
     @abstractmethod
-    def _do_cancel_job(self):
-        ...
+    def _do_cancel_job(self): ...
 
     @abstractmethod
-    def delete(self):
-        ...
+    def delete(self): ...
 
-    @abstractproperty
-    def instance_type(self):
-        ...
+    @property
+    @abstractmethod
+    def instance_type(self): ...
 
-    @abstractproperty
-    def artifact_types(self) -> List[str]:
-        ...
+    @property
+    @abstractmethod
+    def artifact_types(self) -> List[str]: ...
 
     @abstractmethod
-    def _do_get_artifact(self, artifact_key: str) -> str:
-        ...
+    def _do_get_artifact(self, artifact_key: str) -> str: ...
 
     # Base Job properties
 
@@ -382,7 +384,7 @@ class Job(ABC):
         read binary data within a context manager
 
         >>> with job.get_artifact_handle("report_json") as file:
-        ...   print(file.read())
+        ...     print(file.read())
 
         Args:
             artifact_key: Artifact type to download.
@@ -460,29 +462,20 @@ class Job(ABC):
         if report_path:
             report_contents = self._get_report_contents(report_path=report_path)
         else:
-            try:
-                if self.model_type == "gpt_x":
+            for artifact_type in [
+                "report_json",
+                "text_metrics_report_json",
+                "classification_report_json",
+                "regression_report_json",
+            ]:
+                try:
                     report_contents = self._get_report_contents(
-                        artifact_type="text_metrics_report_json"
+                        artifact_type=artifact_type
                     )
-                elif self.model_type == "evaluate":
-                    for artifact_type in [
-                        "report_json",
-                        "classification_report_json",
-                        "regression_report_json",
-                        "text_metrics_report_json",
-                    ]:
-                        report_contents = self._get_report_contents(
-                            artifact_type=artifact_type
-                        )
-                        if report_contents:
-                            break
-                else:
-                    report_contents = self._get_report_contents(
-                        artifact_type="report_json"
-                    )
-            except Exception:
-                pass
+                    if report_contents:
+                        break
+                except Exception:
+                    pass
         return report_contents
 
     def _peek_report(self, report_contents: dict) -> Optional[dict]:
@@ -510,7 +503,7 @@ class Job(ABC):
             report_contents
         )
 
-    def get_report_summary(self, report_path: str = None) -> Optional[dict]:
+    def get_report_summary(self, report_path: Optional[str] = None) -> Optional[dict]:
         """Return a summary of the job results
         Args:
             report_path: If a report_path is passed, that report
@@ -604,7 +597,25 @@ class Job(ABC):
         """Get billing details for the current job."""
         return self._data.get("billing_data", {})
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def container_image(self) -> str:
         """Return the container image for the job."""
         ...
+
+    def _handle_submit_error(self, apix: ApiException) -> None:
+        if "Maximum number of" in str(apix):
+            raise MaxConcurrentJobsException(
+                status=apix.status, reason=apix.reason
+            ) from apix
+        if "cannot run job, credits exhausted" in str(apix).lower():
+            raise CreditExhaustException(
+                status=apix.status, reason=apix.reason
+            ) from apix
+        if "has been discontinued" in str(apix).lower():
+            # For discontinued models, include the `body` to
+            # ensure we give the user a specific error message.
+            exc = DiscontinuedModelException(status=apix.status, reason=apix.reason)
+            exc.body = apix.body
+            raise exc from apix
+        raise apix
