@@ -1,13 +1,14 @@
 """
 Classes and methods for working with Gretel Models
 """
+
 from __future__ import annotations
 
 import copy
 import json
 
 from pathlib import Path
-from typing import Iterator, List, Optional, Type, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Iterator, List, Optional, Type, Union
 
 import requests.exceptions
 import yaml
@@ -16,21 +17,23 @@ from smart_open import open
 
 from gretel_client.cli.utils.parser_utils import (
     DataSourceTypes,
-    ref_data_factory,
     RefData,
     RefDataTypes,
+    ref_data_factory,
 )
-from gretel_client.config import get_logger, RunnerMode
+from gretel_client.config import RunnerMode, get_logger
 from gretel_client.dataframe import _DataFrameT
 from gretel_client.models.config import get_model_type_config
-from gretel_client.projects.common import f, ModelArtifact, NO, YES
+from gretel_client.projects.common import NO, YES, ModelArtifact, f
 from gretel_client.projects.exceptions import (
     GretelJobNotFound,
+    MaxConcurrentJobsException,
     ModelConfigError,
     ModelNotFoundError,
 )
 from gretel_client.projects.jobs import Job, Status
 from gretel_client.projects.records import RecordHandler
+from gretel_client.rest.exceptions import ApiException
 
 if TYPE_CHECKING:
     from gretel_client.projects import Project
@@ -80,7 +83,7 @@ def read_model_config(
 
     Args:
         model_config: This argument may be a string to a file on disk or a Gretel configuration template
-            string such as "synthetics/default". First, this function will treat string input as a
+            string such as "synthetics/navigator-ft". First, this function will treat string input as a
             location on disk and attempt to read the file and parse it as YAML or JSON. If this is
             successful, a dict of the config is returned. If the provided `model_config` str is not
             a file on disk, the function will attempt to resolve the config as a shortcut-path
@@ -185,7 +188,8 @@ class Model(Job):
             - ``ModelConfigError`` if the specified model config is invalid.
             - ``RuntimeError`` if the model is submitted more than once.
             - ``ApiException`` if there is a problem submitting the model to
-                Gretel's api.
+                Gretel's api. If the problem was due specifically to reaching the.
+                limit on concurrent jobs, raises ``MaxConcurrentJobsException`` subclass
         """
         if not self._local_model_config:
             raise ModelConfigError("No model config exists to submit.")
@@ -193,12 +197,23 @@ class Model(Job):
         if self.model_id:
             raise RuntimeError("This model was already submitted.")
 
-        resp = self._projects_api.create_model(
-            project_id=self.project.name,
-            body=self._local_model_config,
-            dry_run=YES if dry_run else NO,
-            runner_mode=runner_mode.api_value,
-        )
+        body = self._local_model_config
+        provenance = self.project.session.context.job_provenance
+        if len(provenance) > 0:
+            body = {
+                "config": body,
+                "provenance": provenance,
+            }
+
+        try:
+            resp = self._projects_api.create_model(
+                project_id=self.project.project_guid,
+                body=body,
+                dry_run=YES if dry_run else NO,
+                runner_mode=runner_mode.api_value,
+            )
+        except ApiException as ex:
+            self._handle_submit_error(ex)
 
         self._data: dict = resp[f.DATA]
         self.worker_key = resp[f.WORKER_KEY]
@@ -349,7 +364,7 @@ class Model(Job):
         """Deletes the remote model."""
         if self.model_id:
             return self._projects_api.delete_model(
-                project_id=self.project.name, model_id=self.model_id
+                project_id=self.project.project_guid, model_id=self.model_id
             )
 
     def validate_data_source(self):
@@ -380,14 +395,14 @@ class Model(Job):
             self.project.default_artifacts_handler.validate_data_source(data_source)
 
     def __repr__(self) -> str:
-        return f"Model(id={self.model_id}, project={self.project.name})"
+        return f"Model(id={self.model_id}, project={self.project.project_guid})"
 
     def _do_get_job_details(self, extra_expand: Optional[list[str]] = None):
         expand = [f.LOGS]
         if extra_expand:
             expand.extend(extra_expand)
         return self._projects_api.get_model(
-            project_id=self.project.name, model_id=self.model_id, expand=expand
+            project_id=self.project.project_guid, model_id=self.model_id, expand=expand
         )
 
     def create_record_handler_obj(
@@ -419,26 +434,25 @@ class Model(Job):
 
     def get_record_handlers(self) -> Iterator[RecordHandler]:
         """Returns a list of record handlers associated with the model."""
-        for status in Status:
-            # Setting the start value for offset in each query, limiting each result set to 10 records
-            offset = 0
-            limit = 10
-            while True:
-                handlers = (
-                    self._projects_api.query_record_handlers(
-                        project_id=self.project.project_id,
-                        model_id=self.model_id,
-                        status=status.value,
-                        skip=offset,
-                        limit=limit,
-                    )
-                    .get("data")
-                    .get("handlers")
+        # Setting the start value for offset in each query, limiting each result set to 10 records
+        offset = 0
+        limit = 10
+        while True:
+            handlers = (
+                self._projects_api.query_record_handlers(
+                    project_id=self.project.project_id,
+                    model_id=self.model_id,
+                    skip=offset,
+                    status=",".join(status.value for status in Status),
+                    limit=limit,
                 )
-                # will break once no more handlers remain in the result set in the last pagination
-                if len(handlers) > 0:
-                    for handler in handlers:
-                        yield RecordHandler(self, record_id=handler["uid"])
-                    offset += limit + 1
-                else:
-                    break
+                .get("data")
+                .get("handlers")
+            )
+            # will break once no more handlers remain in the result set in the last pagination
+            if len(handlers) > 0:
+                for handler in handlers:
+                    yield RecordHandler(self, record_id=handler["uid"])
+                offset += limit
+            else:
+                break
