@@ -1,40 +1,46 @@
 """
 High level API for interacting with a Gretel Project
 """
+
 from __future__ import annotations
 
+import re
+
 from contextlib import contextmanager
-from functools import wraps
+from functools import cached_property, wraps
 from pathlib import Path
 from typing import Any, BinaryIO, Dict, Iterator, List, Optional, Type, TypeVar, Union
 
-from backports.cached_property import cached_property
-
 from gretel_client.cli.utils.parser_utils import (
     DataSourceTypes,
-    ref_data_factory,
     RefDataTypes,
+    ref_data_factory,
 )
 from gretel_client.config import (
     ClientConfig,
+    RunnerMode,
     get_logger,
     get_session_config,
-    RunnerMode,
 )
 from gretel_client.dataframe import _DataFrameT
 from gretel_client.projects.artifact_handlers import (
     ArtifactsHandler,
-    cloud_handler,
     CloudArtifactsHandler,
-    hybrid_handler,
     HybridArtifactsHandler,
+    cloud_handler,
+    hybrid_handler,
 )
 from gretel_client.projects.common import f
 from gretel_client.projects.exceptions import GretelProjectError
 from gretel_client.projects.models import Model
 from gretel_client.rest import models
 from gretel_client.rest.api.projects_api import ProjectsApi
-from gretel_client.rest.exceptions import ForbiddenException, UnauthorizedException
+from gretel_client.rest.exceptions import (
+    ApiException,
+    ForbiddenException,
+    NotFoundException,
+    UnauthorizedException,
+)
 from gretel_client.users.users import get_me
 
 DATA = "data"
@@ -90,6 +96,7 @@ class Project:
         display_name: Optional[str] = None,
         runner_mode: Optional[Union[RunnerMode, str]] = None,
         session: Optional[ClientConfig] = None,
+        cluster_guid: Optional[str] = None,
     ):
         self.client_config = session or get_session_config()
         self.projects_api = self.client_config.get_api(ProjectsApi)
@@ -100,6 +107,7 @@ class Project:
         self.display_name = display_name
         self.runner_mode = RunnerMode.parse_optional(runner_mode)
         self._deleted = False
+        self.cluster_guid = cluster_guid
 
     @property
     def session(self) -> Optional[ClientConfig]:
@@ -125,6 +133,7 @@ class Project:
             display_name=self.display_name,
             runner_mode=self.runner_mode,
             session=session,
+            cluster_guid=self.cluster_guid,
         )
 
     @cached_property
@@ -164,16 +173,13 @@ class Project:
             logger.warning(
                 "``include_models`` is deprecated and will be removed in the 0.9 release."
             )
-        self.projects_api.delete_project(project_id=self.project_id)
+        self.projects_api.delete_project(project_id=self.project_guid)
         self._deleted = True
 
     @check_not_deleted
     def get_console_url(self) -> str:
         """Returns web link to access the project from Gretel's console."""
-        console_base = self.client_config.endpoint.replace("api", "console").replace(
-            "cloud", "ai"
-        )
-        return f"{console_base}/{self.project_guid}"
+        return f"{self.client_config.console_endpoint}/{self.project_guid}"
 
     @property
     def as_dict(self) -> dict:
@@ -200,7 +206,11 @@ class Project:
         self,
         factory: Type[MT] = Model,
         limit: int = 100,
+        skip: Optional[int] = None,
         model_name: str = "",
+        sort_by: Optional[str] = None,
+        sort_field: Optional[str] = None,
+        status: Optional[str] = None,
     ) -> Iterator[MT]:
         """Search for project models.
 
@@ -210,14 +220,26 @@ class Project:
                 is passed, a dictionary representation of the search results
                 will be returned.
             limit: Limits the number of project models to return
+            skip: Number of models to skip before applying limit
             model_name: Name of the model to try and match on (partial match)
+            sort_by: Direction to sort. Uses 'asc' if not provided
+            sort_field: Field to sort on. Uses 'last_modified' if not provided
+            status: Filter to models with specific status.
         """
         if factory not in (dict, Model):
             raise ValueError("factory must be one of ``dict`` or ``Model``.")
 
         api_args = {"project_id": self.name, "limit": limit}
+        if skip is not None:
+            api_args["skip"] = skip
         if model_name:
             api_args["model_name"] = model_name
+        if sort_by:
+            api_args["sort_by"] = sort_by
+        if sort_field:
+            api_args["sort_field"] = sort_field
+        if status:
+            api_args["status"] = status
 
         result = self.projects_api.get_models(**api_args)
         searched_models = result.get(DATA).get(MODELS)
@@ -318,7 +340,7 @@ class Project:
         read binary data within a context manager
 
         >>> with job.get_artifact_handle("report_json") as file:
-        ...   print(file.read())
+        ...     print(file.read())
 
         Args:
             key: Artifact key to download.
@@ -366,6 +388,7 @@ def search_projects(
             project_guid=p.get("guid"),
             desc=p.get("description"),
             display_name=p.get("display_name"),
+            runner_mode=p.get("runner_mode"),
             session=session,
         )
         for p in projects.get(DATA).get(PROJECTS)
@@ -378,6 +401,8 @@ def create_project(
     desc: Optional[str] = None,
     display_name: Optional[str] = None,
     session: Optional[ClientConfig] = None,
+    runner_mode: Optional[Union[RunnerMode, str]] = None,
+    hybrid_environment_guid: Optional[str] = None,
 ) -> Project:
     """
     Excplit project creation. This function will simply call
@@ -389,11 +414,37 @@ def create_project(
 
     payload = {}
     if name:
+        _validate_project_name(name)
         payload["name"] = name
     if desc:
         payload["description"] = desc
     if display_name:
         payload["display_name"] = display_name
+    if runner_mode := RunnerMode.parse_optional(runner_mode):
+        payload["runner_mode"] = runner_mode.value
+    elif runner_mode := RunnerMode.parse_optional(session.default_runner):
+        payload["runner_mode"] = runner_mode.value
+    if hybrid_environment_guid:
+        if runner_mode != RunnerMode.HYBRID:
+            raise ValueError(
+                "cannot specify a hybrid environment for non-hybrid projects"
+            )
+        payload["cluster_guid"] = hybrid_environment_guid
+
+    if name:
+        # ensure name is unique
+        # we need to check this before, because create_project REST call is now idempotent
+        # so we can support retries
+        try:
+            project = api.get_project(project_id=name)
+            if project.get(DATA, {}).get("project", {}).get("_id"):
+                raise ApiException(
+                    status=400, reason=f"Project '{name}' already exists."
+                )
+        except (UnauthorizedException, ForbiddenException, NotFoundException):
+            # In the case we get a 4xx error, proceed to try creating the project
+            # and use that as the failure mode
+            pass
 
     resp = api.create_project(project=models.Project(**payload))
     project = api.get_project(project_id=resp.get(DATA).get("id"))
@@ -406,6 +457,8 @@ def create_project(
         project_guid=proj.get("guid"),
         desc=proj.get("description"),
         display_name=proj.get("display_name"),
+        runner_mode=proj.get("runner_mode"),
+        cluster_guid=proj.get("cluster_guid"),
         session=session,
     )
 
@@ -418,6 +471,7 @@ def get_project(
     display_name: Optional[str] = None,
     runner_mode: Optional[Union[RunnerMode, str]] = None,
     session: Optional[ClientConfig] = None,
+    hybrid_environment_guid: Optional[str] = None,
 ) -> Project:
     """Used to get or create a Gretel project.
 
@@ -452,6 +506,12 @@ def get_project(
             project_args["display_name"] = display_name
         if runner_mode:
             project_args["runner_mode"] = runner_mode.value
+        if hybrid_environment_guid:
+            if runner_mode != RunnerMode.HYBRID:
+                raise ValueError(
+                    "cannot specify a hybrid environment for non-hybrid projects"
+                )
+            project_args["cluster_guid"] = hybrid_environment_guid
 
     if not name and create:
         resp = api.create_project(project=models.Project(**project_args))
@@ -460,9 +520,10 @@ def get_project(
     if name:
         try:
             project = api.get_project(project_id=name)
-        except (UnauthorizedException, ForbiddenException):
+        except (UnauthorizedException, ForbiddenException, NotFoundException):
             if create:
                 project_args["name"] = name
+                _validate_project_name(name)
                 resp = api.create_project(project=models.Project(**project_args))
                 project = api.get_project(project_id=resp.get(DATA).get("id"))
             else:
@@ -486,12 +547,28 @@ def get_project(
         desc=p.get("description"),
         display_name=p.get("display_name"),
         runner_mode=proj_runner_mode,
+        cluster_guid=p.get("cluster_guid"),
         session=session,
     )
 
 
+def _validate_project_name(name: str) -> None:
+    PROJECT_PATTERN = r"^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{5,63}(?<!-)$"
+    if not re.match(PROJECT_PATTERN, name):
+        raise GretelProjectError(
+            f"Project name '{name}' is invalid, it must be "
+            "between 5 and 63 alphanumeric characters and dashes, "
+            "and cannot not start with a dash"
+        ) from None
+
+
 @contextmanager
-def tmp_project(*, session: Optional[ClientConfig] = None):
+def tmp_project(
+    *,
+    session: Optional[ClientConfig] = None,
+    runner_mode: Optional[Union[RunnerMode, str]] = None,
+    hybrid_environment_guid: Optional[str] = None,
+):
     """A temporary project context manager.  Create a new project
     that can be used inside of a "with" statement for temporary purposes.
     The project will be deleted from Gretel Cloud when the scope is exited.
@@ -506,7 +583,12 @@ def tmp_project(*, session: Optional[ClientConfig] = None):
     """
     if session is None:
         session = get_session_config()
-    project = get_project(create=True, session=session)
+    project = get_project(
+        create=True,
+        session=session,
+        runner_mode=runner_mode,
+        hybrid_environment_guid=hybrid_environment_guid,
+    )
     try:
         yield project
     finally:
@@ -519,6 +601,8 @@ def create_or_get_unique_project(
     desc: Optional[str] = None,
     display_name: Optional[str] = None,
     session: Optional[ClientConfig] = None,
+    runner_mode: Optional[Union[RunnerMode, str]] = None,
+    hybrid_environment_guid: Optional[str] = None,
 ) -> Project:
     """
     Helper function that provides a consistent experience for creating
@@ -536,6 +620,10 @@ def create_or_get_unique_project(
             of ``name`` _before_ the user ID is appended.
         session: The client session to use, or ``None`` to use the default client
             session.
+        runner_mode: The runner_mode for the project, one of either
+            ``"cloud"`` or ``"hybrid"``.
+        hybrid_environment_guid: The guid of the hybrid environment to associate
+            with the project, when the runner_mode is of type "hybrid".
 
     NOTE:
         The ``desc`` and ``display_name`` parameters will only be used when
@@ -564,5 +652,7 @@ def create_or_get_unique_project(
         display_name=display_name or name,
         desc=desc,
         session=session,
+        runner_mode=runner_mode,
+        hybrid_environment_guid=hybrid_environment_guid,
     )
     return project
