@@ -1,26 +1,28 @@
 import os
 
 from typing import Callable
-from unittest.mock import ANY, call, MagicMock, patch
+from unittest.mock import ANY, MagicMock, call, patch
 
 import pytest
 
 from click.testing import CliRunner
 
 from gretel_client.cli.cli import cli
+from gretel_client.cli.hybrid import ClusterError
 from gretel_client.cli.utils.parser_utils import RefData
 from gretel_client.config import (
-    ClientConfig,
-    configure_session,
-    get_session_config,
     GRETEL_API_KEY,
     GRETEL_CONFIG_FILE,
     GRETEL_ENDPOINT,
     GRETEL_PROJECT,
+    ClientConfig,
     RunnerMode,
+    configure_session,
+    get_session_config,
 )
-from gretel_client.projects.exceptions import DockerEnvironmentError
 from gretel_client.projects.jobs import Status
+from gretel_client.rest_v1.models.cluster import Cluster
+from gretel_client.rest_v1.models.user_profile import UserProfile
 
 
 @pytest.fixture
@@ -45,7 +47,7 @@ def test_configure_env(write_config: MagicMock, runner: CliRunner):
             GRETEL_CONFIG_FILE: "none",
         },
     ):
-        configure_session(ClientConfig())
+        configure_session(ClientConfig(), validate=False)
         assert get_session_config().api_key == orig_api
         assert get_session_config().endpoint == orig_endpoint
         assert get_session_config().default_project_name == orig_proj
@@ -59,6 +61,7 @@ def test_configure_env(write_config: MagicMock, runner: CliRunner):
                 new_endpoint,
                 "--project",
                 new_proj,
+                "--skip-validate",
             ],
         )
 
@@ -88,58 +91,353 @@ def get_project() -> MagicMock:
         yield get_project
 
 
-@pytest.fixture
-def container_run() -> MagicMock:
-    with patch("gretel_client.cli.models.ContainerRun") as container_run:
-        yield container_run
+@patch("gretel_client.cli.projects.create_project")
+def test_create_nonhybrid_project_warn(create_project: MagicMock, runner: CliRunner):
+    mock_project = MagicMock()
+    mock_project.name = "my-project"
+    mock_project.runner_mode = None
+    create_project.return_value = mock_project
 
-
-def test_local_model_upload_flag(
-    container_run: MagicMock, get_project: MagicMock, runner: CliRunner
-):
+    configure_session(
+        ClientConfig(default_runner="hybrid", artifact_endpoint="s3://my-artifacts"),
+        validate=False,
+    )
     cmd = runner.invoke(
         cli,
         [
-            "models",
+            "projects",
             "create",
-            "--upload-model",
-            "--runner",
-            "local",
-            "--config",
-            "synthetics/default",
-            "--output",
-            "tmp",
-            "--project",
-            "mocked",
+            "--name",
+            "my-project",
         ],
     )
-    print(cmd.output)
     assert cmd.exit_code == 0
-    assert container_run.from_job.return_value.start.call_count == 1
-    assert container_run.from_job.return_value.enable_cloud_uploads.call_count == 1
+    create_project.assert_called_once_with(
+        name="my-project",
+        desc=None,
+        display_name=None,
+        session=ANY,
+        runner_mode=None,
+        hybrid_environment_guid=None,
+    )
+    assert "'--project-type hybrid'" in cmd.stdout
 
 
-def test_local_model_upload_disabled_by_default(
-    container_run: MagicMock, get_project: MagicMock, runner: CliRunner
+@patch("gretel_client.cli.projects.create_project")
+def test_create_nonhybrid_project_no_warn_if_not_hybrid_default_runner(
+    create_project: MagicMock, runner: CliRunner
 ):
+    mock_project = MagicMock()
+    mock_project.name = "my-project"
+    mock_project.runner_mode = None
+    create_project.return_value = mock_project
+
+    configure_session(ClientConfig(default_runner="cloud"), validate=False)
     cmd = runner.invoke(
         cli,
         [
-            "models",
+            "projects",
             "create",
-            "--runner",
-            "local",
-            "--config",
-            "synthetics/default",
-            "--output",
-            "tmp",
-            "--project",
-            "mocked",
+            "--name",
+            "my-project",
         ],
     )
     assert cmd.exit_code == 0
-    assert container_run.from_job.return_value.start.call_count == 1
-    assert container_run.from_job.return_value.enable_cloud_uploads.call_count == 0
+    create_project.assert_called_once_with(
+        name="my-project",
+        desc=None,
+        display_name=None,
+        session=ANY,
+        runner_mode=None,
+        hybrid_environment_guid=None,
+    )
+    assert "'--project-type hybrid'" not in cmd.stdout
+
+
+@patch("gretel_client.cli.projects.create_project")
+def test_create_nonhybrid_project_no_warn_if_explicit_project_type(
+    create_project: MagicMock, runner: CliRunner
+):
+    mock_project = MagicMock()
+    mock_project.name = "my-project"
+    mock_project.runner_mode = None
+    create_project.return_value = mock_project
+
+    configure_session(
+        ClientConfig(default_runner="hybrid", artifact_endpoint="s3://my-artifacts"),
+        validate=False,
+    )
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--name",
+            "my-project",
+            "--project-type",
+            "CLOUD",
+        ],
+    )
+    assert cmd.exit_code == 0
+    create_project.assert_called_once_with(
+        name="my-project",
+        desc=None,
+        display_name=None,
+        session=ANY,
+        runner_mode=RunnerMode.CLOUD,
+        hybrid_environment_guid=None,
+    )
+    assert "'--project-type hybrid'" not in cmd.stdout
+
+
+@patch("gretel_client.cli.projects.create_project")
+def test_create_hybrid_project_no_warn_if_implicit(
+    create_project: MagicMock, runner: CliRunner
+):
+    mock_project = MagicMock()
+    mock_project.name = "my-project"
+    mock_project.runner_mode = RunnerMode.HYBRID
+    create_project.return_value = mock_project
+
+    configure_session(
+        ClientConfig(default_runner="hybrid", artifact_endpoint="s3://my-artifacts"),
+        validate=False,
+    )
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--name",
+            "my-project",
+        ],
+    )
+    assert cmd.exit_code == 0
+    create_project.assert_called_once_with(
+        name="my-project",
+        desc=None,
+        display_name=None,
+        session=ANY,
+        runner_mode=None,
+        hybrid_environment_guid=None,
+    )
+    assert "'--project-type hybrid'" not in cmd.stdout
+
+
+@patch("gretel_client.cli.common.add_session_context")
+def test_create_cloud_project_with_hybrid_env_fails(
+    add_session_context: MagicMock, runner: CliRunner
+):
+    add_session_context.return_value.email = "user@example.com"
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--project-type",
+            "cloud",
+            "--hybrid-environment",
+            "my-hybrid-env",
+        ],
+    )
+    assert cmd.exit_code != 0
+    assert (
+        "A hybrid environment can only be specified for hybrid, not cloud projects"
+        in cmd.output
+    )
+
+
+@patch("gretel_client.cli.common.add_session_context")
+@patch("gretel_client.cli.projects.resolve_hybrid_environment")
+@patch("gretel_client.cli.projects.create_project")
+def test_create_hybrid_project_without_hybrid_env_succeeds(
+    create_project: MagicMock,
+    resolve_hybrid_environment: MagicMock,
+    add_session_context: MagicMock,
+    runner: CliRunner,
+):
+    add_session_context.return_value.email = "user@example.com"
+
+    resolve_hybrid_environment.return_value = Cluster(
+        guid="cluster_1234",
+        name="my-hybrid-env",
+        owner_profile=UserProfile(email="foo@example.com"),
+    )
+
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--project-type",
+            "hybrid",
+        ],
+    )
+    assert cmd.exit_code == 0
+    assert "Creating a hybrid project" not in cmd.output
+    assert (
+        "Automatically selected only available hybrid environment 'my-hybrid-env'"
+        in cmd.output
+    )
+    assert (
+        "The deployment user of the hybrid environment, foo@example.com," in cmd.output
+    )
+
+    resolve_hybrid_environment.assert_called_once_with(
+        ANY, None, allow_auto_select=True
+    )
+    create_project.assert_called_once_with(
+        name=None,
+        desc=None,
+        display_name=None,
+        session=ANY,
+        runner_mode=RunnerMode.HYBRID,
+        hybrid_environment_guid="cluster_1234",
+    )
+
+
+@patch("gretel_client.cli.common.add_session_context")
+@patch("gretel_client.cli.projects.resolve_hybrid_environment")
+@patch("gretel_client.cli.projects.create_project")
+def test_create_project_with_hybrid_env_succeeds(
+    create_project: MagicMock,
+    resolve_hybrid_environment: MagicMock,
+    add_session_context: MagicMock,
+    runner: CliRunner,
+):
+    add_session_context.return_value.email = "user@example.com"
+
+    resolve_hybrid_environment.return_value = Cluster(
+        guid="cluster_1234",
+        name="my-hybrid-env",
+        owner_profile=UserProfile(email="foo@example.com"),
+    )
+
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--hybrid-environment",
+            "my-hybrid-env",
+        ],
+    )
+    assert cmd.exit_code == 0
+    assert "Creating a hybrid project" in cmd.output
+    assert (
+        "The deployment user of the hybrid environment, foo@example.com," in cmd.output
+    )
+
+    resolve_hybrid_environment.assert_called_once_with(
+        ANY, "my-hybrid-env", allow_auto_select=True
+    )
+    create_project.assert_called_once_with(
+        name=None,
+        desc=None,
+        display_name=None,
+        session=ANY,
+        runner_mode=RunnerMode.HYBRID,
+        hybrid_environment_guid="cluster_1234",
+    )
+
+
+@patch("gretel_client.cli.common.add_session_context")
+@patch("gretel_client.cli.projects.resolve_hybrid_environment")
+def test_create_project_with_no_available_hybrid_env_fails_with_none_hint(
+    resolve_hybrid_environment: MagicMock,
+    add_session_context: MagicMock,
+    runner: CliRunner,
+):
+    add_session_context.return_value.email = "user@example.com"
+
+    resolve_hybrid_environment.side_effect = ClusterError(
+        "no hybrid environments are available"
+    )
+
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--project-type",
+            "hybrid",
+        ],
+    )
+    assert cmd.exit_code != 0
+    assert "no hybrid environments are available" in cmd.output
+    assert (
+        "If you wish to create a hybrid project not bound to any hybrid environment"
+        in cmd.output
+    )
+
+
+@patch("gretel_client.cli.common.add_session_context")
+@patch("gretel_client.cli.projects.resolve_hybrid_environment")
+def test_create_project_with_nonexistent_hybrid_env_fails_with_none_hint(
+    resolve_hybrid_environment: MagicMock,
+    add_session_context: MagicMock,
+    runner: CliRunner,
+):
+    add_session_context.return_value.email = "user@example.com"
+
+    resolve_hybrid_environment.side_effect = ClusterError(
+        "no such hybrid environment 'foo'"
+    )
+
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--hybrid-environment",
+            "foo",
+        ],
+    )
+    assert cmd.exit_code != 0
+    assert "Creating a hybrid project" in cmd.output
+    assert "no such hybrid environment 'foo'" in cmd.output
+    assert (
+        "If you wish to create a hybrid project not bound to any hybrid environment"
+        not in cmd.output
+    )
+
+
+@patch("gretel_client.cli.common.add_session_context")
+@patch("gretel_client.cli.projects.create_project")
+@patch("gretel_client.cli.projects.resolve_hybrid_environment")
+def test_create_project_with_none_hybrid_env_succeeds(
+    resolve_hybrid_environment: MagicMock,
+    create_project: MagicMock,
+    add_session_context: MagicMock,
+    runner: CliRunner,
+):
+    add_session_context.return_value.email = "user@example.com"
+
+    create_project.return_value.runner_mode = RunnerMode.HYBRID
+    create_project.return_value.cluster_guid = None
+
+    cmd = runner.invoke(
+        cli,
+        [
+            "projects",
+            "create",
+            "--hybrid-environment",
+            "none",
+        ],
+    )
+    assert cmd.exit_code == 0
+    assert "Creating a hybrid project" in cmd.output
+    assert "for instructions on setting up a hybrid environment." in cmd.output
+
+    resolve_hybrid_environment.assert_not_called()
+    create_project.assert_called_once_with(
+        name=None,
+        desc=None,
+        display_name=None,
+        session=ANY,
+        runner_mode=RunnerMode.HYBRID,
+        hybrid_environment_guid=None,
+    )
 
 
 def test_does_read_model_json(
@@ -312,14 +610,11 @@ def test_prefers_project_from_env(runner: CliRunner, get_project: MagicMock):
 
 
 @patch.dict(os.environ, {"GRETEL_DEFAULT_PROJECT": ""})
-@patch("gretel_client.cli.common.get_session_config")
 def test_default_session_project(
-    get_session_config: MagicMock,
     runner: CliRunner,
     get_project: MagicMock,
 ):
-    get_session_config.return_value.default_runner = "manual"
-    get_session_config.return_value.default_project_name = "default-session-project"
+    get_session_config().update_default_project("default-session-project")
     cmd = runner.invoke(
         cli,
         [
@@ -333,33 +628,6 @@ def test_default_session_project(
     get_project.assert_called_once_with(name="default-session-project", session=ANY)
 
 
-@patch("gretel_client.cli.common.check_docker_env")
-def test_does_gracefully_handle_docker_errors(
-    check_docker_env: MagicMock,
-    runner: CliRunner,
-    get_project: MagicMock,
-):
-    check_docker_env.side_effect = DockerEnvironmentError()
-    cmd = runner.invoke(
-        cli,
-        [
-            "models",
-            "create",
-            "--runner",
-            "local",
-            "--config",
-            "synthetics/default",
-            "--output",
-            "tmp",
-            "--project",
-            "mocked",
-        ],
-    )
-    check_docker_env.assert_called_once()
-    assert cmd.exit_code == 1
-    assert "docker is not running" in cmd.output
-
-
 def test_can_name_model(get_project: MagicMock, runner: CliRunner):
     cmd = runner.invoke(
         cli,
@@ -367,7 +635,7 @@ def test_can_name_model(get_project: MagicMock, runner: CliRunner):
             "models",
             "create",
             "--config",
-            "synthetics/default",
+            "synthetics/navigator-ft",
             "--name",
             "test-project",
         ],
@@ -456,56 +724,6 @@ def test_search_models_with_model_name(
     assert cmd.exit_code == 0
 
 
-@patch.dict(os.environ, {"RUNNER_MODES": "hybrid manual"})
-@patch("gretel_client.cli.agent.get_agent")
-@patch("gretel_client.agents.agent.AgentConfig._update_max_workers")
-def test_get_agent_env_var_passing(
-    max_worker_mock: MagicMock,
-    get_agent_mock: MagicMock,
-    runner: CliRunner,
-):
-    cmd = runner.invoke(
-        cli,
-        [
-            "agent",
-            "start",
-            "--driver",
-            "k8s",
-            "--max-workers",
-            "0",
-        ],
-    )
-    print(cmd.output)
-    get_agent_mock.assert_called_once()
-    args, _ = get_agent_mock.call_args
-    assert [RunnerMode.HYBRID, RunnerMode.MANUAL] == args[0].runner_modes
-    assert cmd.exit_code == 0
-
-
-@patch.dict(os.environ, {"RUNNER_MODES": "hybrid manualy"})
-@patch("gretel_client.cli.agent.get_agent")
-@patch("gretel_client.agents.agent.AgentConfig._update_max_workers")
-def test_get_agent_env_var_passing_fails(
-    max_worker_mock: MagicMock,
-    get_agent_mock: MagicMock,
-    runner: CliRunner,
-):
-    cmd = runner.invoke(
-        cli,
-        [
-            "agent",
-            "start",
-            "--driver",
-            "k8s",
-            "--max-workers",
-            "0",
-        ],
-    )
-    print(cmd.output)
-    get_agent_mock.assert_not_called()
-    assert cmd.exit_code == 1
-
-
 @patch("gretel_client.cli.workflows.get_session_config")
 def test_workflows_read_from_env_manual(
     get_session_config: MagicMock,
