@@ -5,6 +5,7 @@ import tempfile
 from pathlib import Path
 from unittest import mock
 from unittest.mock import MagicMock, patch
+from urllib.parse import unquote_plus
 
 import certifi
 import pytest
@@ -13,20 +14,26 @@ from urllib3 import HTTPResponse
 from urllib3.exceptions import MaxRetryError
 
 from gretel_client.config import (
-    _load_config,
-    ClientConfig,
-    configure_session,
-    get_session_config,
+    CLIENT_METRICS_HEADER_KEY,
     GRETEL_PREVIEW_FEATURES,
     GRETEL_RUNNER_MODE,
+    ClientConfig,
     GretelApiRetry,
     GretelClientConfigurationError,
     PreviewFeatures,
     RunnerMode,
+    _load_config,
+    add_session_context,
+    configure_session,
+    get_client_version,
+    get_session_config,
     write_config,
 )
 from gretel_client.rest.api.projects_api import ProjectsApi
 from gretel_client.rest.exceptions import ForbiddenException
+from gretel_client.rest_v1.models.list_serverless_tenants_response import (
+    ListServerlessTenantsResponse,
+)
 
 
 def test_custom_artifact_endpoint_requires_hybrid_runner():
@@ -69,15 +76,15 @@ def test_does_set_session_factory(dev_ep):
         )
     try:
         assert get_session_config() != config
-        configure_session(config)
+        configure_session(config, validate=False)
         assert get_session_config() == config
     finally:
-        configure_session(_load_config())
+        configure_session(_load_config(), validate=False)
 
 
 @patch.dict(os.environ, {"GRETEL_API_KEY": "grtutest"})
 def test_can_get_api_bindings():
-    configure_session(ClientConfig.from_env())
+    configure_session(ClientConfig.from_env(), validate=False)
 
     client = get_session_config()
     assert isinstance(client.get_api(ProjectsApi), ProjectsApi)
@@ -87,7 +94,7 @@ def test_can_get_api_bindings():
     os.environ, {"GRETEL_API_KEY": "grtutest", "http_proxy": "http://localhost:8080"}
 )
 def test_proxy_set_http():
-    configure_session(ClientConfig.from_env())
+    configure_session(ClientConfig.from_env(), validate=False)
 
     config = get_session_config()
     client = config.get_api(ProjectsApi)
@@ -98,7 +105,7 @@ def test_proxy_set_http():
     os.environ, {"GRETEL_API_KEY": "grtutest", "https_proxy": "https://localhost:8080"}
 )
 def test_proxy_set_https():
-    configure_session(ClientConfig.from_env())
+    configure_session(ClientConfig.from_env(), validate=False)
 
     config = get_session_config()
     client = config.get_api(ProjectsApi)
@@ -114,7 +121,7 @@ def test_proxy_set_https():
     },
 )
 def test_proxy_set_all_proxy():
-    configure_session(ClientConfig.from_env())
+    configure_session(ClientConfig.from_env(), validate=False)
 
     config = get_session_config()
     client = config.get_api(ProjectsApi)
@@ -127,7 +134,8 @@ def test_configure_preview_features():
         {GRETEL_PREVIEW_FEATURES: PreviewFeatures.ENABLED.value},
     ):
         configure_session(
-            _load_config()
+            _load_config(),
+            validate=False,
         )  # ensure the session is reloaded with new env variables
         assert get_session_config().preview_features_enabled
 
@@ -147,14 +155,14 @@ def test_configure_session_with_cache(
     get_pass.return_value = "grtu..."
 
     with mock.patch.dict("os.environ", {}, clear=True):
-        configure_session(api_key="prompt", cache="yes")
+        configure_session(api_key="prompt", cache="yes", validate=False)
     get_pass.assert_called_once()
     write_config.assert_called_once()
     assert write_config.call_args[0][0].api_key == "grtu..."
 
     write_config.reset_mock()
 
-    configure_session(api_key="grtu...")
+    configure_session(api_key="grtu...", validate=False)
     write_config.assert_not_called()
 
 
@@ -162,7 +170,7 @@ def test_configure_session_with_cache(
 def test_clear_gretel_config(_get_config_path: MagicMock):
     _get_config_path.return_value.exists.return_value = False
     with mock.patch.dict("os.environ", {}, clear=True):
-        configure_session(clear=True)
+        configure_session(clear=True, validate=False)
     config_path = _get_config_path.return_value
     config_path.unlink.assert_called_once()
     config_path.parent.rmdir.assert_called_once()
@@ -177,6 +185,7 @@ def test_configure_hybrid_session(_get_config_path, dev_ep):
             endpoint=dev_ep,
             default_runner="hybrid",
             artifact_endpoint="s3://my-bucket",
+            validate=False,
         )
     config = get_session_config()
 
@@ -195,6 +204,7 @@ def test_custom_artifact_endpoint_strips_trailing_slash(_get_config_path, dev_ep
             endpoint=dev_ep,
             default_runner="hybrid",
             artifact_endpoint="s3://my-bucket/",
+            validate=False,
         )
     config = get_session_config()
 
@@ -223,6 +233,7 @@ def test_configure_session_cached_values_plus_overrides(
             artifact_endpoint="s3://bucket-xyz",
             api_key=api_key,
             cache=cache,
+            validate=False,
         )
         config = get_session_config()
 
@@ -269,17 +280,46 @@ def test_configure_session_cached_values_plus_invalid_overrides(
             configure_session(
                 api_key="grtu...",
                 artifact_endpoint="s3://bucket-xyz",
+                validate=False,
             )
 
         tmp.close()
         os.unlink(tmp.name)
 
 
+def test_metrics_headers(dev_ep):
+    def _get_headers(api):
+        return api.api_client.default_headers
+
+    def get_metrics_header(session):
+        api = session.get_api(ProjectsApi)
+        return unquote_plus(_get_headers(api)[CLIENT_METRICS_HEADER_KEY])
+
+    configure_session(api_key="grtu...", validate=False)
+
+    common = f"python_sdk_version={get_client_version()}"
+
+    # Always include the sdk version
+    assert get_metrics_header(get_session_config()) == common
+
+    # Tagged sessions append extra metadata
+    tagged = add_session_context(session=None, client_metrics={"hello": "world"})
+    assert get_metrics_header(tagged) == f"{common};hello=world"
+
+    # Tagged sessions can be composed
+    multi_tagged = add_session_context(session=tagged, client_metrics={"foo": "bar"})
+    assert get_metrics_header(multi_tagged) == f"{common};hello=world;foo=bar"
+
+    # add_session_context and resultant Tagged sessions are None-safe
+    no_extra_metrics = add_session_context()
+    assert get_metrics_header(no_extra_metrics) == common
+
+
 @patch("urllib3.PoolManager")
 @patch.dict(os.environ, {"GRETEL_API_KEY": "grtutest"})
 def test_defaults_to_certifi_certs(pool_manager: MagicMock):
     config = ClientConfig.from_env()
-    client = config.get_api(ProjectsApi)
+    config.get_api(ProjectsApi)
 
     _, kwargs = pool_manager.call_args
     assert kwargs.get("ca_certs") == certifi.where()
@@ -353,3 +393,83 @@ def test_retry_throttling_403s():
         )
 
     assert e.value.reason == error
+
+
+# Test when a user configures an enterprise tenant that it should call set_serverless_api
+def test_configure_enterprise():
+    cfg = ClientConfig(
+        endpoint="localhost:8080",
+        api_key="grtu...",
+        tenant_name="enterprise",
+    )
+    with patch.object(ClientConfig, "set_serverless_api") as set_serverless_api:
+        with patch.object(ClientConfig, "email"):
+            set_serverless_api.return_value = True
+            configure_session(cfg)
+            set_serverless_api.assert_called_once()
+
+
+# Test when tenant name is provided and validate is false, should throw a client config error
+def test_configure_enterprise_validate_false():
+    cfg = ClientConfig(
+        endpoint="localhost:8080",
+        api_key="grtu...",
+        tenant_name="enterprise",
+    )
+    with patch.object(ClientConfig, "set_serverless_api") as set_serverless_api:
+        with patch.object(ClientConfig, "email"):
+            set_serverless_api.return_value = True
+            with pytest.raises(GretelClientConfigurationError):
+                configure_session(cfg, validate=False)
+            set_serverless_api.assert_not_called
+
+
+# Test when a list is returned and a matching tenant name is found
+def test_set_serverless_api_tenant_found_match():
+    cfg = ClientConfig(
+        endpoint="localhost:8080",
+        api_key="grtu...",
+        tenant_name="enterprise",
+    )
+    cfg.get_v1_api = MagicMock()
+    cfg.get_v1_api.return_value = MagicMock()
+    data = '{"tenants":[{"guid":"1","domain_guid":"fake","created_at":"2024-11-18T17:45:27.189+00:00","cloud_provider":{"provider":"AWS","region":"us-west-2"},"config":{"cell_id":"1","api_endpoint":"gretelhost:8080"},"name":"enterprise"}]}'
+    cfg.get_v1_api.return_value.list_serverless_tenants.return_value = (
+        ListServerlessTenantsResponse.from_json(data)
+    )
+    assert cfg.set_serverless_api()
+    assert cfg.endpoint == "https://gretelhost:8080"
+
+
+# Test when no tenants are returned
+def test_set_serverless_api_tenant_no_tenants():
+    cfg = ClientConfig(
+        endpoint="localhost:8080",
+        api_key="grtu...",
+        tenant_name="enterprise",
+    )
+    cfg.get_v1_api = MagicMock()
+    cfg.get_v1_api.return_value = MagicMock()
+    data = '{"tenants":[]}'
+    cfg.get_v1_api.return_value.list_serverless_tenants.return_value = (
+        ListServerlessTenantsResponse.from_json(data)
+    )
+    assert not cfg.set_serverless_api()
+    assert cfg.endpoint == "localhost:8080"
+
+
+# Test when a list is returned but no matching tenant name
+def test_set_serverless_api_tenant_no_match():
+    cfg = ClientConfig(
+        endpoint="localhost:8080",
+        api_key="grtu...",
+        tenant_name="enterprise",
+    )
+    cfg.get_v1_api = MagicMock()
+    cfg.get_v1_api.return_value = MagicMock()
+    data = '{"tenants":[{"guid":"1","domain_guid":"fake","created_at":"2024-11-18T17:45:27.189+00:00","cloud_provider":{"provider":"AWS","region":"us-west-2"},"config":{"cell_id":"1","api_endpoint":"gretelhost:8080"},"name":"other"}]}'
+    cfg.get_v1_api.return_value.list_serverless_tenants.return_value = (
+        ListServerlessTenantsResponse.from_json(data)
+    )
+    assert not cfg.set_serverless_api()
+    assert cfg.endpoint == "localhost:8080"
